@@ -30,23 +30,24 @@ import java.util.concurrent.Future;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
 
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.hbase.DoNotRetryIOException;
 import org.apache.hadoop.hbase.HRegionInfo;
 import org.apache.hadoop.hbase.RegionLocations;
 import org.apache.hadoop.hbase.TableName;
-import org.apache.hadoop.hbase.classification.InterfaceAudience;
 import org.apache.hadoop.hbase.client.ScannerCallable.MoreResults;
 import org.apache.hadoop.hbase.util.Pair;
+import org.apache.yetus.audience.InterfaceAudience;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 /**
  * This class has the logic for handling scanners for regions with and without replicas.
- * 1. A scan is attempted on the default (primary) region
- * 2. The scanner sends all the RPCs to the default region until it is done, or, there
- * is a timeout on the default (a timeout of zero is disallowed).
- * 3. If there is a timeout in (2) above, scanner(s) is opened on the non-default replica(s)
+ * 1. A scan is attempted on the default (primary) region, or a specific region.
+ * 2. The scanner sends all the RPCs to the default/specific region until it is done, or, there
+ * is a timeout on the default/specific region (a timeout of zero is disallowed).
+ * 3. If there is a timeout in (2) above, scanner(s) is opened on the non-default replica(s) only
+ *    for Consistency.TIMELINE without specific replica id specified.
  * 4. The results from the first successful scanner are taken, and it is stored which server
  * returned the results.
  * 5. The next RPCs are done on the above stored server until it is done or there is a timeout,
@@ -55,7 +56,7 @@ import org.apache.hadoop.hbase.util.Pair;
  */
 @InterfaceAudience.Private
 class ScannerCallableWithReplicas implements RetryingCallable<Result[]> {
-  private static final Log LOG = LogFactory.getLog(ScannerCallableWithReplicas.class);
+  private static final Logger LOG = LoggerFactory.getLogger(ScannerCallableWithReplicas.class);
   volatile ScannerCallable currentScannerCallable;
   AtomicBoolean replicaSwitched = new AtomicBoolean(false);
   final ClusterConnection cConnection;
@@ -68,7 +69,7 @@ class ScannerCallableWithReplicas implements RetryingCallable<Result[]> {
   private final TableName tableName;
   private Configuration conf;
   private int scannerTimeout;
-  private Set<ScannerCallable> outstandingCallables = new HashSet<ScannerCallable>();
+  private Set<ScannerCallable> outstandingCallables = new HashSet<>();
   private boolean someRPCcancelled = false; //required for testing purposes only
   private int regionReplication = 0;
 
@@ -125,7 +126,7 @@ class ScannerCallableWithReplicas implements RetryingCallable<Result[]> {
   }
 
   @Override
-  public Result [] call(int timeout) throws IOException {
+  public Result[] call(int timeout) throws IOException {
     // If the active replica callable was closed somewhere, invoke the RPC to
     // really close it. In the case of regular scanners, this applies. We make couple
     // of RPCs to a RegionServer, and when that region is exhausted, we set
@@ -158,13 +159,13 @@ class ScannerCallableWithReplicas implements RetryingCallable<Result[]> {
       RegionLocations rl = null;
       try {
         rl = RpcRetryingCallerWithReadReplicas.getRegionLocations(true,
-            RegionReplicaUtil.DEFAULT_REPLICA_ID, cConnection, tableName,
+          RegionReplicaUtil.DEFAULT_REPLICA_ID, cConnection, tableName,
             currentScannerCallable.getRow());
       } catch (RetriesExhaustedException | DoNotRetryIOException e) {
         // We cannot get the primary replica region location, it is possible that the region server
         // hosting meta table is down, it needs to proceed to try cached replicas directly.
-        if (cConnection instanceof ConnectionManager.HConnectionImplementation) {
-          rl = ((ConnectionManager.HConnectionImplementation) cConnection)
+        if (cConnection instanceof ConnectionImplementation) {
+          rl = ((ConnectionImplementation) cConnection)
               .getCachedLocation(tableName, currentScannerCallable.getRow());
           if (rl == null) {
             throw e;
@@ -179,13 +180,13 @@ class ScannerCallableWithReplicas implements RetryingCallable<Result[]> {
     // allocate a boundedcompletion pool of some multiple of number of replicas.
     // We want to accomodate some RPCs for redundant replica scans (but are still in progress)
     ResultBoundedCompletionService<Pair<Result[], ScannerCallable>> cs =
-        new ResultBoundedCompletionService<Pair<Result[], ScannerCallable>>(
+        new ResultBoundedCompletionService<>(
             RpcRetryingCallerFactory.instantiate(ScannerCallableWithReplicas.this.conf), pool,
             regionReplication * 5);
 
     AtomicBoolean done = new AtomicBoolean(false);
     replicaSwitched.set(false);
-    // submit call for the primary replica.
+    // submit call for the primary replica or user specified replica
     addCallsForCurrentReplica(cs);
     int startIndex = 0;
 
@@ -207,13 +208,13 @@ class ScannerCallableWithReplicas implements RetryingCallable<Result[]> {
         LOG.debug("Scan with primary region returns " + e.getCause());
       }
 
-      // If rl's size is 1 or scan's consitency is strong, it needs to throw
-      // out the exception from the primary replica
-      if ((regionReplication == 1) || (scan.getConsistency() == Consistency.STRONG)) {
+      // If rl's size is 1 or scan's consitency is strong, or scan is over specific replica,
+      // it needs to throw out the exception from the primary replica
+      if (regionReplication == 1 || scan.getConsistency() == Consistency.STRONG ||
+        scan.getReplicaId() >= 0) {
         // Rethrow the first exception
         RpcRetryingCallerWithReadReplicas.throwEnrichedException(e, retries);
       }
-
       startIndex = 1;
     } catch (CancellationException e) {
       throw new InterruptedIOException(e.getMessage());
@@ -223,8 +224,9 @@ class ScannerCallableWithReplicas implements RetryingCallable<Result[]> {
 
     // submit call for the all of the secondaries at once
     int endIndex = regionReplication;
-    if (scan.getConsistency() == Consistency.STRONG) {
-      // When scan's consistency is strong, do not send to the secondaries
+    if (scan.getConsistency() == Consistency.STRONG || scan.getReplicaId() >= 0) {
+      // When scan's consistency is strong or scan is over specific replica region, do not send to
+      // the secondaries
       endIndex = 1;
     } else {
       // TODO: this may be an overkill for large region replication
@@ -301,7 +303,6 @@ class ScannerCallableWithReplicas implements RetryingCallable<Result[]> {
   /**
    * When a scanner switches in the middle of scanning (the 'next' call fails
    * for example), the upper layer {@link ClientScanner} needs to know
-   * @return
    */
   public boolean switchedToADifferentReplica() {
     return replicaSwitched.get();
@@ -392,7 +393,7 @@ class ScannerCallableWithReplicas implements RetryingCallable<Result[]> {
         return null;
       }
       Result[] res = this.caller.callWithoutRetries(this.callable, callTimeout);
-      return new Pair<Result[], ScannerCallable>(res, this.callable);
+      return new Pair<>(res, this.callable);
     }
 
     @Override
@@ -425,8 +426,8 @@ class ScannerCallableWithReplicas implements RetryingCallable<Result[]> {
     public void cancel() {
       cancelled = true;
       caller.cancel();
-      if (callable.getController() != null) {
-        callable.getController().startCancel();
+      if (callable.getRpcController() != null) {
+        callable.getRpcController().startCancel();
       }
       someRPCcancelled = true;
     }
