@@ -19,19 +19,26 @@ package org.apache.hadoop.hbase.util;
 
 import static org.junit.Assert.assertArrayEquals;
 import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 
 import java.math.BigDecimal;
 import java.util.Arrays;
 import java.util.Collections;
-
+import org.apache.hadoop.hbase.HBaseClassTestRule;
+import org.apache.hadoop.hbase.testclassification.MiscTests;
 import org.apache.hadoop.hbase.testclassification.SmallTests;
+import org.junit.ClassRule;
 import org.junit.Test;
 import org.junit.experimental.categories.Category;
 
-@Category(SmallTests.class)
+@Category({MiscTests.class, SmallTests.class})
 public class TestOrderedBytes {
 
+  @ClassRule
+  public static final HBaseClassTestRule CLASS_RULE =
+      HBaseClassTestRule.forClass(TestOrderedBytes.class);
+
   // integer constants for testing Numeric code paths
   static final Long[] I_VALS =
     { 0L, 1L, 10L, 99L, 100L, 1234L, 9999L, 10000L, 10001L, 12345L, 123450L, Long.MAX_VALUE,
@@ -1256,32 +1263,50 @@ public class TestOrderedBytes {
     int cnt = 0;
     PositionedByteRange buff = new SimplePositionedMutableByteRange(1024);
     for (Order ord : new Order[] { Order.ASCENDING, Order.DESCENDING }) {
-      int o;
-      o = OrderedBytes.encodeNull(buff, ord); cnt++;
-      o = OrderedBytes.encodeNumeric(buff, negInf, ord); cnt++;
-      o = OrderedBytes.encodeNumeric(buff, negLarge, ord); cnt++;
-      o = OrderedBytes.encodeNumeric(buff, negMed, ord); cnt++;
-      o = OrderedBytes.encodeNumeric(buff, negSmall, ord); cnt++;
-      o = OrderedBytes.encodeNumeric(buff, zero, ord); cnt++;
-      o = OrderedBytes.encodeNumeric(buff, posSmall, ord); cnt++;
-      o = OrderedBytes.encodeNumeric(buff, posMed, ord); cnt++;
-      o = OrderedBytes.encodeNumeric(buff, posLarge, ord); cnt++;
-      o = OrderedBytes.encodeNumeric(buff, posInf, ord); cnt++;
-      o = OrderedBytes.encodeNumeric(buff, nan, ord); cnt++;
-      o = OrderedBytes.encodeInt8(buff, int8, ord); cnt++;
-      o = OrderedBytes.encodeInt16(buff, int16, ord); cnt++;
-      o = OrderedBytes.encodeInt32(buff, int32, ord); cnt++;
-      o = OrderedBytes.encodeInt64(buff, int64, ord); cnt++;
-      o = OrderedBytes.encodeFloat32(buff, float32, ord); cnt++;
-      o = OrderedBytes.encodeFloat64(buff, float64, ord); cnt++;
-      o = OrderedBytes.encodeString(buff, text, ord); cnt++;
-      o = OrderedBytes.encodeBlobVar(buff, blobVar, ord); cnt++;
+      OrderedBytes.encodeNull(buff, ord);
+      cnt++;
+      OrderedBytes.encodeNumeric(buff, negInf, ord);
+      cnt++;
+      OrderedBytes.encodeNumeric(buff, negLarge, ord);
+      cnt++;
+      OrderedBytes.encodeNumeric(buff, negMed, ord);
+      cnt++;
+      OrderedBytes.encodeNumeric(buff, negSmall, ord);
+      cnt++;
+      OrderedBytes.encodeNumeric(buff, zero, ord);
+      cnt++;
+      OrderedBytes.encodeNumeric(buff, posSmall, ord);
+      cnt++;
+      OrderedBytes.encodeNumeric(buff, posMed, ord);
+      cnt++;
+      OrderedBytes.encodeNumeric(buff, posLarge, ord);
+      cnt++;
+      OrderedBytes.encodeNumeric(buff, posInf, ord);
+      cnt++;
+      OrderedBytes.encodeNumeric(buff, nan, ord);
+      cnt++;
+      OrderedBytes.encodeInt8(buff, int8, ord);
+      cnt++;
+      OrderedBytes.encodeInt16(buff, int16, ord);
+      cnt++;
+      OrderedBytes.encodeInt32(buff, int32, ord);
+      cnt++;
+      OrderedBytes.encodeInt64(buff, int64, ord);
+      cnt++;
+      OrderedBytes.encodeFloat32(buff, float32, ord);
+      cnt++;
+      OrderedBytes.encodeFloat64(buff, float64, ord);
+      cnt++;
+      OrderedBytes.encodeString(buff, text, ord);
+      cnt++;
+      OrderedBytes.encodeBlobVar(buff, blobVar, ord);
+      cnt++;
     }
 
     buff.setPosition(0);
     assertEquals(OrderedBytes.length(buff), cnt);
     for (int i = 0; i < cnt; i++) {
-      assertEquals(true, OrderedBytes.isEncodedValue(buff));
+      assertTrue(OrderedBytes.isEncodedValue(buff));
       OrderedBytes.skip(buff);
     }
   }
