@@ -26,6 +26,7 @@ import java.io.IOException;
 import java.util.Collections;
 import java.util.HashSet;
 import java.util.List;
+import java.util.Map;
 import java.util.Optional;
 import java.util.Set;
 import java.util.stream.Collectors;
@@ -80,18 +81,19 @@ import org.apache.hadoop.hbase.protobuf.generated.RSGroupAdminProtos.RemoveServe
 import org.apache.hadoop.hbase.protobuf.generated.RSGroupAdminProtos.RemoveServersResponse;
 import org.apache.hadoop.hbase.protobuf.generated.RSGroupAdminProtos.RenameRSGroupRequest;
 import org.apache.hadoop.hbase.protobuf.generated.RSGroupAdminProtos.RenameRSGroupResponse;
+import org.apache.hadoop.hbase.protobuf.generated.RSGroupAdminProtos.UpdateRSGroupConfigRequest;
+import org.apache.hadoop.hbase.protobuf.generated.RSGroupAdminProtos.UpdateRSGroupConfigResponse;
 import org.apache.hadoop.hbase.protobuf.generated.TableProtos;
 import org.apache.hadoop.hbase.security.User;
 import org.apache.hadoop.hbase.security.UserProvider;
 import org.apache.hadoop.hbase.security.access.AccessChecker;
 import org.apache.hadoop.hbase.security.access.Permission.Action;
+import org.apache.hbase.thirdparty.com.google.common.collect.Maps;
+import org.apache.hbase.thirdparty.com.google.common.collect.Sets;
 import org.apache.yetus.audience.InterfaceAudience;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import org.apache.hbase.thirdparty.com.google.common.annotations.VisibleForTesting;
-import org.apache.hbase.thirdparty.com.google.common.collect.Sets;
-
 // TODO: Encapsulate MasterObserver functions into separate subclass.
 @CoreCoprocessor
 @InterfaceAudience.Private
@@ -147,7 +149,6 @@ public class RSGroupAdminEndpoint implements MasterCoprocessor, MasterObserver {
     return groupInfoManager;
   }
 
-  @VisibleForTesting
   RSGroupAdminServer getGroupAdminServer() {
     return groupAdminServer;
   }
@@ -429,6 +430,29 @@ public class RSGroupAdminEndpoint implements MasterCoprocessor, MasterObserver {
       }
       done.run(builder.build());
     }
+
+    @Override
+    public void updateRSGroupConfig(RpcController controller, UpdateRSGroupConfigRequest request,
+                                    RpcCallback<UpdateRSGroupConfigResponse> done) {
+      UpdateRSGroupConfigResponse.Builder builder = UpdateRSGroupConfigResponse.newBuilder();
+      String groupName = request.getGroupName();
+      Map<String, String> configuration = Maps.newHashMap();
+      request.getConfigurationList().forEach(p -> configuration.put(p.getName(), p.getValue()));
+      LOG.info("{} update rsgroup {} configuration {}", master.getClientIdAuditPrefix(), groupName,
+          configuration);
+      try {
+        if (master.getMasterCoprocessorHost() != null) {
+          master.getMasterCoprocessorHost().preUpdateRSGroupConfig(groupName, configuration);
+        }
+        groupAdminServer.updateRSGroupConfig(groupName, configuration);
+        if (master.getMasterCoprocessorHost() != null) {
+          master.getMasterCoprocessorHost().postUpdateRSGroupConfig(groupName, configuration);
+        }
+      } catch (IOException e) {
+        CoprocessorRpcUtils.setControllerException(controller, e);
+      }
+      done.run(builder.build());
+    }
   }
 
   boolean rsgroupHasServersOnline(TableDescriptor desc) throws IOException {
@@ -461,7 +485,7 @@ public class RSGroupAdminEndpoint implements MasterCoprocessor, MasterObserver {
   }
 
   void assignTableToGroup(TableDescriptor desc) throws IOException {
-    RSGroupInfo rsGroupInfo = groupInfoManager.determineRSGroupInfoForTable(desc.getTableName());
+    RSGroupInfo rsGroupInfo = determineRSGroupInfoForTable(desc);
     if (rsGroupInfo == null) {
       throw new ConstraintException("Default RSGroup for this table " + desc.getTableName()
         + " does not exist.");
@@ -484,21 +508,75 @@ public class RSGroupAdminEndpoint implements MasterCoprocessor, MasterObserver {
     if (desc.getTableName().isSystemTable()) {
       return;
     }
-    RSGroupInfo rsGroupInfo = groupInfoManager.determineRSGroupInfoForTable(desc.getTableName());
-    if (rsGroupInfo == null) {
-      throw new ConstraintException("Default RSGroup for this table " + desc.getTableName()
-        + " does not exist.");
-    }
-    if (!RSGroupUtil.rsGroupHasOnlineServer(master, rsGroupInfo)) {
-      throw new HBaseIOException("No online servers in the rsgroup " + rsGroupInfo.getName()
-        + " which table " + desc.getTableName().getNameAsString() + " belongs to");
-    }
-    synchronized (groupInfoManager) {
-      groupInfoManager.moveTables(
-        Collections.singleton(desc.getTableName()), rsGroupInfo.getName());
+    moveTableToValidRSGroup(desc);
+  }
+
+  @Override
+  public void preModifyTableAction(ObserverContext<MasterCoprocessorEnvironment> ctx,
+    TableName tableName, TableDescriptor currentDescriptor, TableDescriptor newDescriptor)
+    throws IOException {
+    // If table's rsgroup is changed, it must be valid
+    if (!currentDescriptor.getRegionServerGroup().equals(newDescriptor.getRegionServerGroup())) {
+      RSGroupInfo rsGroupInfo = determineRSGroupInfoForTable(newDescriptor);
+      validateRSGroup(newDescriptor, rsGroupInfo);
     }
   }
 
+  @Override
+  public void postCompletedModifyTableAction(ObserverContext<MasterCoprocessorEnvironment> ctx,
+    TableName tableName, TableDescriptor oldDescriptor, TableDescriptor currentDescriptor)
+    throws IOException {
+    // If table's rsgroup is changed, move table into the rsgroup.
+    if (!oldDescriptor.getRegionServerGroup().equals(currentDescriptor.getRegionServerGroup())) {
+      RSGroupInfo rsGroupInfo = determineRSGroupInfoForTable(currentDescriptor);
+      moveTableToRSGroup(currentDescriptor, rsGroupInfo);
+    }
+  }
+
+  // Determine and validate rs group then move table to this valid rs group.
+  private void moveTableToValidRSGroup(TableDescriptor desc) throws IOException {
+    RSGroupInfo rsGroupInfo = determineRSGroupInfoForTable(desc);
+    validateRSGroup(desc, rsGroupInfo);
+    moveTableToRSGroup(desc, rsGroupInfo);
+  }
+
+  private void validateRSGroup(TableDescriptor desc, RSGroupInfo rsGroupInfo) throws IOException {
+    if (rsGroupInfo == null) {
+      throw new ConstraintException(
+        "Default RSGroup for this table " + desc.getTableName() + " does not exist.");
+    }
+    if (!RSGroupUtil.rsGroupHasOnlineServer(master, rsGroupInfo)) {
+      throw new HBaseIOException(
+        "No online servers in the rsgroup " + rsGroupInfo.getName() + " which table " + desc
+          .getTableName().getNameAsString() + " belongs to");
+    }
+  }
+
+  private void moveTableToRSGroup(final TableDescriptor desc, RSGroupInfo rsGroupInfo)
+    throws IOException {
+    // In case of modify table, when rs group is not changed, move is not required.
+    if (!rsGroupInfo.containsTable(desc.getTableName())) {
+      synchronized (groupInfoManager) {
+        groupInfoManager
+          .moveTables(Collections.singleton(desc.getTableName()), rsGroupInfo.getName());
+      }
+    }
+  }
+
+  private RSGroupInfo determineRSGroupInfoForTable(final TableDescriptor desc) throws IOException {
+    Optional<String> optGroupNameOfTable = desc.getRegionServerGroup();
+    if (optGroupNameOfTable.isPresent()) {
+      final RSGroupInfo rsGroup = groupInfoManager.getRSGroup(optGroupNameOfTable.get());
+      if (rsGroup == null) {
+        // When rs group is set in table descriptor then it must exist
+        throw new ConstraintException(
+            "Region server group " + optGroupNameOfTable.get() + " does not exist.");
+      } else {
+        return rsGroup;
+      }
+    }
+    return groupInfoManager.determineRSGroupInfoForTable(desc.getTableName());
+  }
   // Remove table from its RSGroup.
   @Override
   public void postDeleteTable(ObserverContext<MasterCoprocessorEnvironment> ctx,
