@@ -1,6 +1,4 @@
 /**
- * Copyright The Apache Software Foundation
- *
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -33,16 +31,17 @@ import java.util.Map;
 import java.util.Set;
 import java.util.SortedSet;
 
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
-import org.apache.hadoop.hbase.HColumnDescriptor;
-import org.apache.hadoop.hbase.HTableDescriptor;
+import org.apache.hadoop.hbase.HBaseClassTestRule;
 import org.apache.hadoop.hbase.NamespaceDescriptor;
 import org.apache.hadoop.hbase.ServerName;
 import org.apache.hadoop.hbase.TableExistsException;
 import org.apache.hadoop.hbase.TableName;
 import org.apache.hadoop.hbase.TableNotFoundException;
 import org.apache.hadoop.hbase.Waiter;
+import org.apache.hadoop.hbase.client.ColumnFamilyDescriptor;
+import org.apache.hadoop.hbase.client.ColumnFamilyDescriptorBuilder;
+import org.apache.hadoop.hbase.client.TableDescriptor;
+import org.apache.hadoop.hbase.client.TableDescriptorBuilder;
 import org.apache.hadoop.hbase.constraint.ConstraintException;
 import org.apache.hadoop.hbase.master.ServerManager;
 import org.apache.hadoop.hbase.master.TableNamespaceManager;
@@ -56,14 +55,22 @@ import org.junit.AfterClass;
 import org.junit.Assert;
 import org.junit.Before;
 import org.junit.BeforeClass;
+import org.junit.ClassRule;
 import org.junit.Test;
 import org.junit.experimental.categories.Category;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
-import com.google.common.collect.Sets;
+import org.apache.hbase.thirdparty.com.google.common.collect.Sets;
 
-@Category({MediumTests.class})
+@Category({ MediumTests.class })
 public class TestRSGroupsAdmin1 extends TestRSGroupsBase {
-  protected static final Log LOG = LogFactory.getLog(TestRSGroupsAdmin1.class);
+
+  @ClassRule
+  public static final HBaseClassTestRule CLASS_RULE =
+    HBaseClassTestRule.forClass(TestRSGroupsAdmin1.class);
+
+  protected static final Logger LOG = LoggerFactory.getLogger(TestRSGroupsAdmin1.class);
 
   @BeforeClass
   public static void setUp() throws Exception {
@@ -87,19 +94,19 @@ public class TestRSGroupsAdmin1 extends TestRSGroupsBase {
 
   @Test
   public void testValidGroupNames() throws IOException {
-    String[] badNames = {"foo*","foo@","-"};
-    String[] goodNames = {"foo_123"};
+    String[] badNames = { "foo*", "foo@", "-" };
+    String[] goodNames = { "foo_123" };
 
-    for(String entry: badNames) {
+    for (String entry : badNames) {
       try {
         rsGroupAdmin.addRSGroup(entry);
-        fail("Expected a constraint exception for: "+entry);
-      } catch(ConstraintException ex) {
-        //expected
+        fail("Expected a constraint exception for: " + entry);
+      } catch (ConstraintException ex) {
+        // expected
       }
     }
 
-    for(String entry: goodNames) {
+    for (String entry : goodNames) {
       rsGroupAdmin.addRSGroup(entry);
     }
   }
@@ -107,72 +114,71 @@ public class TestRSGroupsAdmin1 extends TestRSGroupsBase {
   @Test
   public void testBogusArgs() throws Exception {
     assertNull(rsGroupAdmin.getRSGroupInfoOfTable(TableName.valueOf("nonexistent")));
-    assertNull(rsGroupAdmin.getRSGroupOfServer(Address.fromParts("bogus",123)));
+    assertNull(rsGroupAdmin.getRSGroupOfServer(Address.fromParts("bogus", 123)));
     assertNull(rsGroupAdmin.getRSGroupInfo("bogus"));
 
     try {
       rsGroupAdmin.removeRSGroup("bogus");
       fail("Expected removing bogus group to fail");
-    } catch(ConstraintException ex) {
-      //expected
+    } catch (ConstraintException ex) {
+      // expected
     }
 
     try {
       rsGroupAdmin.moveTables(Sets.newHashSet(TableName.valueOf("bogustable")), "bogus");
       fail("Expected move with bogus group to fail");
-    } catch(ConstraintException | TableNotFoundException ex) {
-      //expected
+    } catch (ConstraintException | TableNotFoundException ex) {
+      // expected
     }
 
     try {
-      rsGroupAdmin.moveServers(Sets.newHashSet(Address.fromParts("bogus",123)), "bogus");
+      rsGroupAdmin.moveServers(Sets.newHashSet(Address.fromParts("bogus", 123)), "bogus");
       fail("Expected move with bogus group to fail");
-    } catch(ConstraintException ex) {
-      //expected
+    } catch (ConstraintException ex) {
+      // expected
     }
 
     try {
+      admin.balancerSwitch(true, true);
       rsGroupAdmin.balanceRSGroup("bogus");
+      admin.balancerSwitch(false, true);
       fail("Expected move with bogus group to fail");
-    } catch(ConstraintException ex) {
-      //expected
+    } catch (ConstraintException ex) {
+      // expected
     }
   }
 
   @Test
   public void testNamespaceConstraint() throws Exception {
-    String nsName = tablePrefix+"_foo";
-    String groupName = tablePrefix+"_foo";
+    String nsName = tablePrefix + "_foo";
+    String groupName = tablePrefix + "_foo";
     LOG.info("testNamespaceConstraint");
     rsGroupAdmin.addRSGroup(groupName);
     assertTrue(observer.preAddRSGroupCalled);
     assertTrue(observer.postAddRSGroupCalled);
+
     admin.createNamespace(NamespaceDescriptor.create(nsName)
-        .addConfiguration(RSGroupInfo.NAMESPACE_DESC_PROP_GROUP, groupName)
-        .build());
-    //test removing a referenced group
+      .addConfiguration(RSGroupInfo.NAMESPACE_DESC_PROP_GROUP, groupName).build());
+    // test removing a referenced group
     try {
       rsGroupAdmin.removeRSGroup(groupName);
       fail("Expected a constraint exception");
     } catch (IOException ex) {
     }
-    //test modify group
-    //changing with the same name is fine
-    admin.modifyNamespace(
-        NamespaceDescriptor.create(nsName)
-          .addConfiguration(RSGroupInfo.NAMESPACE_DESC_PROP_GROUP, groupName)
-          .build());
-    String anotherGroup = tablePrefix+"_anotherGroup";
+    // test modify group
+    // changing with the same name is fine
+    admin.modifyNamespace(NamespaceDescriptor.create(nsName)
+      .addConfiguration(RSGroupInfo.NAMESPACE_DESC_PROP_GROUP, groupName).build());
+    String anotherGroup = tablePrefix + "_anotherGroup";
     rsGroupAdmin.addRSGroup(anotherGroup);
-    //test add non-existent group
+    // test add non-existent group
     admin.deleteNamespace(nsName);
     rsGroupAdmin.removeRSGroup(groupName);
     assertTrue(observer.preRemoveRSGroupCalled);
     assertTrue(observer.postRemoveRSGroupCalled);
     try {
       admin.createNamespace(NamespaceDescriptor.create(nsName)
-          .addConfiguration(RSGroupInfo.NAMESPACE_DESC_PROP_GROUP, "foo")
-          .build());
+        .addConfiguration(RSGroupInfo.NAMESPACE_DESC_PROP_GROUP, "foo").build());
       fail("Expected a constraint exception");
     } catch (IOException ex) {
     }
@@ -180,8 +186,8 @@ public class TestRSGroupsAdmin1 extends TestRSGroupsBase {
 
   @Test
   public void testGroupInfoMultiAccessing() throws Exception {
-    RSGroupInfoManager manager = RSGroupAdminEndpoint.getGroupInfoManager();
-    final RSGroupInfo defaultGroup = manager.getRSGroup("default");
+    RSGroupInfoManager manager = rsGroupAdminEndpoint.getGroupInfoManager();
+    RSGroupInfo defaultGroup = manager.getRSGroup("default");
     // getRSGroup updates default group's server list
     // this process must not affect other threads iterating the list
     Iterator<Address> it = defaultGroup.getServers().iterator();
@@ -191,32 +197,29 @@ public class TestRSGroupsAdmin1 extends TestRSGroupsBase {
 
   @Test
   public void testFailRemoveGroup() throws IOException, InterruptedException {
-    LOG.info("testFailRemoveGroup");
-
     int initNumGroups = rsGroupAdmin.listRSGroups().size();
-    addGroup(rsGroupAdmin, "bar", 3);
-    TableName tableName = TableName.valueOf(tablePrefix+"_my_table");
+    addGroup("bar", 3);
     TEST_UTIL.createTable(tableName, Bytes.toBytes("f"));
     rsGroupAdmin.moveTables(Sets.newHashSet(tableName), "bar");
     RSGroupInfo barGroup = rsGroupAdmin.getRSGroupInfo("bar");
-    //group is not empty therefore it should fail
+    // group is not empty therefore it should fail
     try {
       rsGroupAdmin.removeRSGroup(barGroup.getName());
       fail("Expected remove group to fail");
-    } catch(IOException e) {
+    } catch (IOException e) {
     }
-    //group cannot lose all it's servers therefore it should fail
+    // group cannot lose all it's servers therefore it should fail
     try {
       rsGroupAdmin.moveServers(barGroup.getServers(), RSGroupInfo.DEFAULT_GROUP);
       fail("Expected move servers to fail");
-    } catch(IOException e) {
+    } catch (IOException e) {
     }
 
     rsGroupAdmin.moveTables(barGroup.getTables(), RSGroupInfo.DEFAULT_GROUP);
     try {
       rsGroupAdmin.removeRSGroup(barGroup.getName());
       fail("Expected move servers to fail");
-    } catch(IOException e) {
+    } catch (IOException e) {
     }
 
     rsGroupAdmin.moveServers(barGroup.getServers(), RSGroupInfo.DEFAULT_GROUP);
@@ -227,13 +230,11 @@ public class TestRSGroupsAdmin1 extends TestRSGroupsBase {
 
   @Test
   public void testMultiTableMove() throws Exception {
-    LOG.info("testMultiTableMove");
-
-    final TableName tableNameA = TableName.valueOf(tablePrefix + "_testMultiTableMoveA");
-    final TableName tableNameB = TableName.valueOf(tablePrefix + "_testMultiTableMoveB");
+    final TableName tableNameA = TableName.valueOf(tablePrefix + name.getMethodName() + "A");
+    final TableName tableNameB = TableName.valueOf(tablePrefix + name.getMethodName() + "B");
     final byte[] familyNameBytes = Bytes.toBytes("f");
-    String newGroupName = getGroupName("testMultiTableMove");
-    final RSGroupInfo newGroup = addGroup(rsGroupAdmin, newGroupName, 1);
+    String newGroupName = getGroupName(name.getMethodName());
+    final RSGroupInfo newGroup = addGroup(newGroupName, 1);
 
     TEST_UTIL.createTable(tableNameA, familyNameBytes);
     TEST_UTIL.createTable(tableNameB, familyNameBytes);
@@ -248,8 +249,8 @@ public class TestRSGroupsAdmin1 extends TestRSGroupsBase {
         if (regionsB == null) {
           return false;
         }
-        return getTableRegionMap().get(tableNameA).size() >= 1
-                && getTableRegionMap().get(tableNameB).size() >= 1;
+        return getTableRegionMap().get(tableNameA).size() >= 1 &&
+          getTableRegionMap().get(tableNameB).size() >= 1;
       }
     });
 
@@ -258,24 +259,24 @@ public class TestRSGroupsAdmin1 extends TestRSGroupsBase {
 
     RSGroupInfo tableGrpB = rsGroupAdmin.getRSGroupInfoOfTable(tableNameB);
     assertTrue(tableGrpB.getName().equals(RSGroupInfo.DEFAULT_GROUP));
-    //change table's group
+    // change table's group
     LOG.info("Moving table [" + tableNameA + "," + tableNameB + "] to " + newGroup.getName());
     rsGroupAdmin.moveTables(Sets.newHashSet(tableNameA, tableNameB), newGroup.getName());
 
-    //verify group change
+    // verify group change
     Assert.assertEquals(newGroup.getName(),
-            rsGroupAdmin.getRSGroupInfoOfTable(tableNameA).getName());
+      rsGroupAdmin.getRSGroupInfoOfTable(tableNameA).getName());
 
     Assert.assertEquals(newGroup.getName(),
-            rsGroupAdmin.getRSGroupInfoOfTable(tableNameB).getName());
+      rsGroupAdmin.getRSGroupInfoOfTable(tableNameB).getName());
 
-    //verify tables' not exist in old group
+    // verify tables' not exist in old group
     Set<TableName> DefaultTables =
-        rsGroupAdmin.getRSGroupInfo(RSGroupInfo.DEFAULT_GROUP).getTables();
+      rsGroupAdmin.getRSGroupInfo(RSGroupInfo.DEFAULT_GROUP).getTables();
     assertFalse(DefaultTables.contains(tableNameA));
     assertFalse(DefaultTables.contains(tableNameB));
 
-    //verify tables' exist in new group
+    // verify tables' exist in new group
     Set<TableName> newGroupTables = rsGroupAdmin.getRSGroupInfo(newGroupName).getTables();
     assertTrue(newGroupTables.contains(tableNameA));
     assertTrue(newGroupTables.contains(tableNameB));
@@ -283,20 +284,19 @@ public class TestRSGroupsAdmin1 extends TestRSGroupsBase {
 
   @Test
   public void testTableMoveTruncateAndDrop() throws Exception {
-    LOG.info("testTableMove");
-
-    final TableName tableName = TableName.valueOf(tablePrefix + "_testTableMoveAndDrop");
     final byte[] familyNameBytes = Bytes.toBytes("f");
-    String newGroupName = getGroupName("testTableMove");
-    final RSGroupInfo newGroup = addGroup(rsGroupAdmin, newGroupName, 2);
+    String newGroupName = getGroupName(name.getMethodName());
+    final RSGroupInfo newGroup = addGroup(newGroupName, 2);
 
     TEST_UTIL.createMultiRegionTable(tableName, familyNameBytes, 5);
     TEST_UTIL.waitFor(WAIT_TIMEOUT, new Waiter.Predicate<Exception>() {
       @Override
       public boolean evaluate() throws Exception {
         List<String> regions = getTableRegionMap().get(tableName);
-        if (regions == null)
+        if (regions == null) {
           return false;
+        }
+
         return getTableRegionMap().get(tableName).size() >= 5;
       }
     });
@@ -304,13 +304,13 @@ public class TestRSGroupsAdmin1 extends TestRSGroupsBase {
     RSGroupInfo tableGrp = rsGroupAdmin.getRSGroupInfoOfTable(tableName);
     assertTrue(tableGrp.getName().equals(RSGroupInfo.DEFAULT_GROUP));
 
-    //change table's group
-    LOG.info("Moving table "+tableName+" to "+newGroup.getName());
+    // change table's group
+    LOG.info("Moving table " + tableName + " to " + newGroup.getName());
     rsGroupAdmin.moveTables(Sets.newHashSet(tableName), newGroup.getName());
 
-    //verify group change
+    // verify group change
     Assert.assertEquals(newGroup.getName(),
-        rsGroupAdmin.getRSGroupInfoOfTable(tableName).getName());
+      rsGroupAdmin.getRSGroupInfoOfTable(tableName).getName());
 
     TEST_UTIL.waitFor(WAIT_TIMEOUT, new Waiter.Predicate<Exception>() {
       @Override
@@ -328,14 +328,14 @@ public class TestRSGroupsAdmin1 extends TestRSGroupsBase {
       }
     });
 
-    //test truncate
+    // test truncate
     admin.disableTable(tableName);
     admin.truncateTable(tableName, true);
     Assert.assertEquals(1, rsGroupAdmin.getRSGroupInfo(newGroup.getName()).getTables().size());
-    Assert.assertEquals(tableName, rsGroupAdmin.getRSGroupInfo(
-        newGroup.getName()).getTables().first());
+    Assert.assertEquals(tableName,
+      rsGroupAdmin.getRSGroupInfo(newGroup.getName()).getTables().first());
 
-    //verify removed table is removed from group
+    // verify removed table is removed from group
     TEST_UTIL.deleteTable(tableName);
     Assert.assertEquals(0, rsGroupAdmin.getRSGroupInfo(newGroup.getName()).getTables().size());
 
@@ -345,10 +345,9 @@ public class TestRSGroupsAdmin1 extends TestRSGroupsBase {
 
   @Test
   public void testDisabledTableMove() throws Exception {
-    final TableName tableName = TableName.valueOf(tablePrefix + "_testDisabledTableMove");
     final byte[] familyNameBytes = Bytes.toBytes("f");
-    String newGroupName = getGroupName("testDisabledTableMove");
-    final RSGroupInfo newGroup = addGroup(rsGroupAdmin, newGroupName, 2);
+    String newGroupName = getGroupName(name.getMethodName());
+    final RSGroupInfo newGroup = addGroup(newGroupName, 2);
 
     TEST_UTIL.createMultiRegionTable(tableName, familyNameBytes, 5);
     TEST_UTIL.waitFor(WAIT_TIMEOUT, new Waiter.Predicate<Exception>() {
@@ -365,16 +364,46 @@ public class TestRSGroupsAdmin1 extends TestRSGroupsBase {
     RSGroupInfo tableGrp = rsGroupAdmin.getRSGroupInfoOfTable(tableName);
     assertTrue(tableGrp.getName().equals(RSGroupInfo.DEFAULT_GROUP));
 
-    //test disable table
+    // test disable table
     admin.disableTable(tableName);
 
-    //change table's group
-    LOG.info("Moving table "+ tableName + " to " + newGroup.getName());
+    // change table's group
+    LOG.info("Moving table " + tableName + " to " + newGroup.getName());
     rsGroupAdmin.moveTables(Sets.newHashSet(tableName), newGroup.getName());
 
-    //verify group change
+    // verify group change
     Assert.assertEquals(newGroup.getName(),
-        rsGroupAdmin.getRSGroupInfoOfTable(tableName).getName());
+      rsGroupAdmin.getRSGroupInfoOfTable(tableName).getName());
+  }
+
+  @Test
+  public void testNonExistentTableMove() throws Exception {
+    TableName tableName = TableName.valueOf(tablePrefix + name.getMethodName());
+
+    RSGroupInfo tableGrp = rsGroupAdmin.getRSGroupInfoOfTable(tableName);
+    assertNull(tableGrp);
+
+    // test if table exists already.
+    boolean exist = admin.tableExists(tableName);
+    assertFalse(exist);
+
+    LOG.info("Moving table " + tableName + " to " + RSGroupInfo.DEFAULT_GROUP);
+    try {
+      rsGroupAdmin.moveTables(Sets.newHashSet(tableName), RSGroupInfo.DEFAULT_GROUP);
+      fail("Table " + tableName + " shouldn't have been successfully moved.");
+    } catch (IOException ex) {
+      assertTrue(ex instanceof TableNotFoundException);
+    }
+
+    try {
+      rsGroupAdmin.moveServersAndTables(Sets.newHashSet(Address.fromParts("bogus", 123)),
+        Sets.newHashSet(tableName), RSGroupInfo.DEFAULT_GROUP);
+      fail("Table " + tableName + " shouldn't have been successfully moved.");
+    } catch (IOException ex) {
+      assertTrue(ex instanceof TableNotFoundException);
+    }
+    // verify group change
+    assertNull(rsGroupAdmin.getRSGroupInfoOfTable(tableName));
   }
 
   @Test
@@ -382,40 +411,39 @@ public class TestRSGroupsAdmin1 extends TestRSGroupsBase {
     toggleQuotaCheckAndRestartMiniCluster(true);
     String nsp = "np1";
     NamespaceDescriptor nspDesc =
-        NamespaceDescriptor.create(nsp).addConfiguration(TableNamespaceManager.KEY_MAX_REGIONS, "5")
-            .addConfiguration(TableNamespaceManager.KEY_MAX_TABLES, "2").build();
+      NamespaceDescriptor.create(nsp).addConfiguration(TableNamespaceManager.KEY_MAX_REGIONS, "5")
+        .addConfiguration(TableNamespaceManager.KEY_MAX_TABLES, "2").build();
     admin.createNamespace(nspDesc);
     assertEquals(3, admin.listNamespaceDescriptors().length);
-    HColumnDescriptor fam1 = new HColumnDescriptor("fam1");
-    HTableDescriptor tableDescOne =
-        new HTableDescriptor(TableName.valueOf(nsp + TableName.NAMESPACE_DELIM + "table1"));
-    tableDescOne.addFamily(fam1);
+    ColumnFamilyDescriptor fam1 = ColumnFamilyDescriptorBuilder.of("fam1");
+    TableDescriptor tableDescOne = TableDescriptorBuilder
+      .newBuilder(TableName.valueOf(nsp + TableName.NAMESPACE_DELIM + "table1"))
+      .setColumnFamily(fam1).build();
     admin.createTable(tableDescOne);
 
-    HTableDescriptor tableDescTwo =
-        new HTableDescriptor(TableName.valueOf(nsp + TableName.NAMESPACE_DELIM + "table2"));
-    tableDescTwo.addFamily(fam1);
+    TableDescriptor tableDescTwo = TableDescriptorBuilder
+      .newBuilder(TableName.valueOf(nsp + TableName.NAMESPACE_DELIM + "table2"))
+      .setColumnFamily(fam1).build();
     boolean constraintViolated = false;
 
     try {
-      admin.createTable(tableDescTwo, Bytes.toBytes("AAA"), Bytes.toBytes("ZZZ"),
-          6);
+      admin.createTable(tableDescTwo, Bytes.toBytes("AAA"), Bytes.toBytes("ZZZ"), 6);
       Assert.fail("Creation table should fail because of quota violation.");
     } catch (Exception exp) {
       assertTrue(exp instanceof IOException);
       constraintViolated = true;
     } finally {
       assertTrue("Constraint not violated for table " + tableDescTwo.getTableName(),
-          constraintViolated);
+        constraintViolated);
     }
     List<RSGroupInfo> rsGroupInfoList = rsGroupAdmin.listRSGroups();
     boolean foundTable2 = false;
     boolean foundTable1 = false;
-    for (int i = 0; i < rsGroupInfoList.size(); i++){
-      if(rsGroupInfoList.get(i).getTables().contains(tableDescTwo.getTableName())){
+    for (int i = 0; i < rsGroupInfoList.size(); i++) {
+      if (rsGroupInfoList.get(i).getTables().contains(tableDescTwo.getTableName())) {
         foundTable2 = true;
       }
-      if(rsGroupInfoList.get(i).getTables().contains(tableDescOne.getTableName())){
+      if (rsGroupInfoList.get(i).getTables().contains(tableDescOne.getTableName())) {
         foundTable1 = true;
       }
     }
@@ -450,11 +478,11 @@ public class TestRSGroupsAdmin1 extends TestRSGroupsBase {
       @Override
       public boolean evaluate() throws Exception {
         return
-            (master.getMasterProcedureExecutor().getActiveExecutorCount() == 0);
+                (master.getMasterProcedureExecutor().getActiveExecutorCount() == 0);
       }
     });
     SortedSet<TableName> tables
-        = rsGroupAdmin.getRSGroupInfo(RSGroupInfo.DEFAULT_GROUP).getTables();
+            = rsGroupAdmin.getRSGroupInfo(RSGroupInfo.DEFAULT_GROUP).getTables();
     assertTrue("Table 't1' must be in 'default' rsgroup", tables.contains(tn1));
 
     // Cleanup
@@ -465,9 +493,8 @@ public class TestRSGroupsAdmin1 extends TestRSGroupsBase {
     TEST_UTIL.shutdownMiniCluster();
     TEST_UTIL.getConfiguration().setBoolean(QuotaUtil.QUOTA_CONF_KEY, enable);
     TEST_UTIL.startMiniCluster(NUM_SLAVES_BASE - 1);
-    TEST_UTIL.getConfiguration().setInt(
-        ServerManager.WAIT_ON_REGIONSERVERS_MINTOSTART,
-        NUM_SLAVES_BASE - 1);
+    TEST_UTIL.getConfiguration().setInt(ServerManager.WAIT_ON_REGIONSERVERS_MINTOSTART,
+      NUM_SLAVES_BASE - 1);
     TEST_UTIL.getConfiguration().setBoolean(SnapshotManager.HBASE_SNAPSHOT_ENABLED, true);
     initialize();
   }
@@ -475,16 +502,13 @@ public class TestRSGroupsAdmin1 extends TestRSGroupsBase {
   @Test
   public void testRenameRSGroup() throws Exception {
     // Add rsgroup, and assign 2 servers and a table to it.
-    RSGroupInfo oldgroup = addGroup(rsGroupAdmin, "oldgroup", 2);
+    RSGroupInfo oldgroup = addGroup("oldgroup", 2);
     final TableName tb1 = TableName.valueOf("testRename");
     TEST_UTIL.createTable(tb1, "tr");
     rsGroupAdmin.moveTables(Sets.newHashSet(tb1), oldgroup.getName());
-    TEST_UTIL.waitFor(1000, new Waiter.Predicate<Exception>() {
-      @Override
-      public boolean evaluate() throws Exception {
-        return rsGroupAdmin.getRSGroupInfoOfTable(tb1).getServers().size() == 2;
-      }
-    });
+    TEST_UTIL.waitFor(1000,
+      (Waiter.Predicate<Exception>) () ->
+        rsGroupAdmin.getRSGroupInfoOfTable(tb1).getServers().size() == 2);
     oldgroup = rsGroupAdmin.getRSGroupInfo(oldgroup.getName());
     assertEquals(2, oldgroup.getServers().size());
     assertEquals(oldgroup.getName(), rsGroupAdmin.getRSGroupInfoOfTable(tb1).getName());
@@ -492,16 +516,13 @@ public class TestRSGroupsAdmin1 extends TestRSGroupsBase {
 
     // Another rsgroup and table for verification
     // that they are unchanged during we're renaming oldgroup.
-    RSGroupInfo normal = addGroup(rsGroupAdmin, "normal", 1);
+    RSGroupInfo normal = addGroup("normal", 1);
     final TableName tb2 = TableName.valueOf("unmovedTable");
     TEST_UTIL.createTable(tb2, "ut");
     rsGroupAdmin.moveTables(Sets.newHashSet(tb2), normal.getName());
-    TEST_UTIL.waitFor(1000, new Waiter.Predicate<Exception>() {
-      @Override
-      public boolean evaluate() throws Exception {
-        return rsGroupAdmin.getRSGroupInfoOfTable(tb2).getServers().size() == 1;
-      }
-    });
+    TEST_UTIL.waitFor(1000,
+      (Waiter.Predicate<Exception>) () ->
+        rsGroupAdmin.getRSGroupInfoOfTable(tb2).getServers().size() == 1);
     normal = rsGroupAdmin.getRSGroupInfo(normal.getName());
     assertEquals(1, normal.getServers().size());
     assertEquals(normal.getName(), rsGroupAdmin.getRSGroupInfoOfTable(tb2).getName());
@@ -525,18 +546,26 @@ public class TestRSGroupsAdmin1 extends TestRSGroupsBase {
   public void testRenameRSGroupConstraints() throws Exception {
     // Add RSGroup, and assign 2 servers and a table to it.
     String oldGroupName = "oldGroup";
-    RSGroupInfo oldGroup = addGroup(rsGroupAdmin, oldGroupName, 2);
+    RSGroupInfo oldGroup = addGroup(oldGroupName, 2);
     oldGroup = rsGroupAdmin.getRSGroupInfo(oldGroup.getName());
     assertNotNull(oldGroup);
     assertEquals(2, oldGroup.getServers().size());
 
     //Add another RSGroup
     String anotherRSGroupName = "anotherRSGroup";
-    RSGroupInfo anotherGroup = addGroup(rsGroupAdmin, anotherRSGroupName, 1);
+    RSGroupInfo anotherGroup = addGroup(anotherRSGroupName, 1);
     anotherGroup = rsGroupAdmin.getRSGroupInfo(anotherGroup.getName());
     assertNotNull(anotherGroup);
     assertEquals(1, anotherGroup.getServers().size());
 
+    //Rename a non existing RSGroup
+    try {
+      rsGroupAdmin.renameRSGroup("nonExistingRSGroup", "newRSGroup1");
+      fail("ConstraintException was expected.");
+    } catch (ConstraintException e) {
+      assertTrue(e.getMessage().contains("does not exist"));
+    }
+
     //Rename to existing group
     try {
       rsGroupAdmin.renameRSGroup(oldGroup.getName(), anotherRSGroupName);
