@@ -1,5 +1,4 @@
-/*
- *
+/**
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -16,14 +15,17 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.hadoop.hbase.io;
 
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertTrue;
+
 import java.io.IOException;
 import java.lang.management.ManagementFactory;
 import java.lang.management.RuntimeMXBean;
 import java.nio.ByteBuffer;
 import java.util.ArrayList;
+import java.util.LinkedList;
 import java.util.Map;
 import java.util.TreeMap;
 import java.util.concurrent.ConcurrentHashMap;
@@ -33,40 +35,62 @@ import java.util.concurrent.CopyOnWriteArraySet;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.atomic.AtomicLong;
+import java.util.concurrent.atomic.AtomicReference;
 import java.util.concurrent.locks.ReentrantReadWriteLock;
-
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.apache.hadoop.hbase.HBaseClassTestRule;
 import org.apache.hadoop.hbase.KeyValue;
-import org.apache.hadoop.hbase.testclassification.SmallTests;
 import org.apache.hadoop.hbase.client.Delete;
+import org.apache.hadoop.hbase.client.Mutation;
 import org.apache.hadoop.hbase.client.Put;
 import org.apache.hadoop.hbase.io.hfile.BlockCacheKey;
-import org.apache.hadoop.hbase.io.hfile.LruCachedBlock;
+import org.apache.hadoop.hbase.io.hfile.ExclusiveMemHFileBlock;
+import org.apache.hadoop.hbase.io.hfile.HFileBlock;
+import org.apache.hadoop.hbase.io.hfile.HFileContext;
 import org.apache.hadoop.hbase.io.hfile.LruBlockCache;
-import org.apache.hadoop.hbase.regionserver.CellSkipListSet;
+import org.apache.hadoop.hbase.io.hfile.LruCachedBlock;
+import org.apache.hadoop.hbase.io.hfile.SharedMemHFileBlock;
+import org.apache.hadoop.hbase.regionserver.CSLMImmutableSegment;
+import org.apache.hadoop.hbase.regionserver.CellArrayImmutableSegment;
+import org.apache.hadoop.hbase.regionserver.CellArrayMap;
+import org.apache.hadoop.hbase.regionserver.CellSet;
+import org.apache.hadoop.hbase.regionserver.CompactingMemStore;
+import org.apache.hadoop.hbase.regionserver.CompactionPipeline;
 import org.apache.hadoop.hbase.regionserver.DefaultMemStore;
 import org.apache.hadoop.hbase.regionserver.HRegion;
 import org.apache.hadoop.hbase.regionserver.HStore;
-import org.apache.hadoop.hbase.regionserver.TimeRangeTracker;
+import org.apache.hadoop.hbase.regionserver.ImmutableSegment;
+import org.apache.hadoop.hbase.regionserver.MemStoreCompactor;
+import org.apache.hadoop.hbase.regionserver.MutableSegment;
+import org.apache.hadoop.hbase.regionserver.Segment;
+import org.apache.hadoop.hbase.regionserver.StoreContext;
+import org.apache.hadoop.hbase.regionserver.TimeRangeTracker.NonSyncTimeRangeTracker;
+import org.apache.hadoop.hbase.regionserver.TimeRangeTracker.SyncTimeRangeTracker;
+import org.apache.hadoop.hbase.regionserver.throttle.StoreHotnessProtector;
+import org.apache.hadoop.hbase.testclassification.IOTests;
+import org.apache.hadoop.hbase.testclassification.SmallTests;
 import org.apache.hadoop.hbase.util.ClassSize;
 import org.junit.BeforeClass;
+import org.junit.ClassRule;
 import org.junit.Test;
 import org.junit.experimental.categories.Category;
-
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertTrue;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 /**
  * Testing the sizing that HeapSize offers and compares to the size given by
  * ClassSize.
  */
-@Category(SmallTests.class)
+@Category({IOTests.class, SmallTests.class})
 public class TestHeapSize  {
-  private static final Log LOG = LogFactory.getLog(TestHeapSize.class);
+
+  @ClassRule
+  public static final HBaseClassTestRule CLASS_RULE =
+      HBaseClassTestRule.forClass(TestHeapSize.class);
+
+  private static final Logger LOG = LoggerFactory.getLogger(TestHeapSize.class);
   // List of classes implementing HeapSize
   // BatchOperation, BatchUpdate, BlockIndex, Entry, Entry<K,V>, HStoreKey
-  // KeyValue, LruBlockCache, LruHashMap<K,V>, Put, WALKey
+  // KeyValue, LruBlockCache, Put, WALKey
 
   @BeforeClass
   public static void beforeClass() throws Exception {
@@ -173,6 +197,15 @@ public class TestHeapSize  {
       assertEquals(expected, actual);
     }
 
+    // CellArrayMap
+    cl = CellArrayMap.class;
+    expected = ClassSize.estimateBase(cl, false);
+    actual = ClassSize.CELL_ARRAY_MAP;
+    if(expected != actual) {
+      ClassSize.estimateBase(cl, true);
+      assertEquals(expected, actual);
+    }
+
     // ReentrantReadWriteLock
     cl = ReentrantReadWriteLock.class;
     expected = ClassSize.estimateBase(cl, false);
@@ -227,19 +260,28 @@ public class TestHeapSize  {
       assertEquals(expected, actual);
     }
 
-    // TimeRangeTracker
-    cl = TimeRangeTracker.class;
+    // SyncTimeRangeTracker
+    cl = SyncTimeRangeTracker.class;
     expected = ClassSize.estimateBase(cl, false);
-    actual = ClassSize.TIMERANGE_TRACKER;
+    actual = ClassSize.SYNC_TIMERANGE_TRACKER;
     if (expected != actual) {
       ClassSize.estimateBase(cl, true);
       assertEquals(expected, actual);
     }
 
-    // CellSkipListSet
-    cl = CellSkipListSet.class;
+    // NonSyncTimeRangeTracker
+    cl = NonSyncTimeRangeTracker.class;
     expected = ClassSize.estimateBase(cl, false);
-    actual = ClassSize.CELL_SKIPLIST_SET;
+    actual = ClassSize.NON_SYNC_TIMERANGE_TRACKER;
+    if (expected != actual) {
+      ClassSize.estimateBase(cl, true);
+      assertEquals(expected, actual);
+    }
+
+    // CellSet
+    cl = CellSet.class;
+    expected = ClassSize.estimateBase(cl, false);
+    actual = ClassSize.CELL_SET;
     if (expected != actual) {
       ClassSize.estimateBase(cl, true);
       assertEquals(expected, actual);
@@ -304,22 +346,133 @@ public class TestHeapSize  {
     // DefaultMemStore Deep Overhead
     actual = DefaultMemStore.DEEP_OVERHEAD;
     expected = ClassSize.estimateBase(cl, false);
-    expected += (2 * ClassSize.estimateBase(AtomicLong.class, false));
-    expected += ClassSize.estimateBase(AtomicInteger.class, false);
-    expected += (2 * ClassSize.estimateBase(CellSkipListSet.class, false));
-    expected += (2 * ClassSize.estimateBase(ConcurrentSkipListMap.class, false));
-    expected += (2 * ClassSize.estimateBase(TimeRangeTracker.class, false));
+    if (expected != actual) {
+      ClassSize.estimateBase(cl, true);
+      assertEquals(expected, actual);
+    }
+
+    // CompactingMemStore Deep Overhead
+    cl = CompactingMemStore.class;
+    actual = CompactingMemStore.DEEP_OVERHEAD;
+    expected = ClassSize.estimateBase(cl, false);
+    expected += ClassSize.estimateBase(AtomicBoolean.class, false);
+    expected += ClassSize.estimateBase(AtomicBoolean.class, false);
+    expected += ClassSize.estimateBase(CompactionPipeline.class, false);
+    expected += ClassSize.estimateBase(LinkedList.class, false); //inside CompactionPipeline
+    expected += ClassSize.estimateBase(LinkedList.class, false); //inside CompactionPipeline
+    expected += ClassSize.estimateBase(MemStoreCompactor.class, false);
+    expected += ClassSize.estimateBase(AtomicBoolean.class, false);// inside MemStoreCompactor
+    if (expected != actual) {
+      ClassSize.estimateBase(cl, true);
+      ClassSize.estimateBase(AtomicBoolean.class, true);
+      ClassSize.estimateBase(AtomicBoolean.class, true);
+      ClassSize.estimateBase(CompactionPipeline.class, true);
+      ClassSize.estimateBase(LinkedList.class, true);
+      ClassSize.estimateBase(LinkedList.class, true);
+      ClassSize.estimateBase(MemStoreCompactor.class, true);
+      ClassSize.estimateBase(AtomicBoolean.class, true);
+      assertEquals(expected, actual);
+    }
+
+    // Segment Deep overhead
+    cl = Segment.class;
+    actual = Segment.DEEP_OVERHEAD;
+    expected = ClassSize.estimateBase(cl, false);
+    expected += 2 * ClassSize.estimateBase(AtomicLong.class, false);
+    expected += ClassSize.estimateBase(AtomicReference.class, false);
+    expected += ClassSize.estimateBase(CellSet.class, false);
+    expected += ClassSize.estimateBase(ReentrantReadWriteLock.class, false);
+    if (expected != actual) {
+      ClassSize.estimateBase(cl, true);
+      ClassSize.estimateBase(AtomicLong.class, true);
+      ClassSize.estimateBase(AtomicReference.class, true);
+      ClassSize.estimateBase(CellSet.class, true);
+      ClassSize.estimateBase(ReentrantReadWriteLock.class,true);
+      assertEquals(expected, actual);
+    }
+
+    // MutableSegment Deep overhead
+    cl = MutableSegment.class;
+    actual = MutableSegment.DEEP_OVERHEAD;
+    expected = ClassSize.estimateBase(cl, false);
+    expected += 2 * ClassSize.estimateBase(AtomicLong.class, false);
+    expected += ClassSize.estimateBase(AtomicReference.class, false);
+    expected += ClassSize.estimateBase(CellSet.class, false);
+    expected += ClassSize.estimateBase(ReentrantReadWriteLock.class, false);
+    expected += ClassSize.estimateBase(SyncTimeRangeTracker.class, false);
+    expected += ClassSize.estimateBase(ConcurrentSkipListMap.class, false);
+    expected += ClassSize.estimateBase(AtomicBoolean.class, false);
     if (expected != actual) {
       ClassSize.estimateBase(cl, true);
       ClassSize.estimateBase(AtomicLong.class, true);
       ClassSize.estimateBase(AtomicLong.class, true);
-      ClassSize.estimateBase(AtomicInteger.class, true);
-      ClassSize.estimateBase(CellSkipListSet.class, true);
-      ClassSize.estimateBase(CellSkipListSet.class, true);
+      ClassSize.estimateBase(AtomicReference.class, true);
+      ClassSize.estimateBase(CellSet.class, true);
+      ClassSize.estimateBase(ReentrantReadWriteLock.class,true);
+      ClassSize.estimateBase(SyncTimeRangeTracker.class, true);
       ClassSize.estimateBase(ConcurrentSkipListMap.class, true);
+      ClassSize.estimateBase(AtomicBoolean.class,true);
+      assertEquals(expected, actual);
+    }
+
+    // ImmutableSegments Deep overhead
+    cl = ImmutableSegment.class;
+    actual = ImmutableSegment.DEEP_OVERHEAD;
+    expected = ClassSize.estimateBase(cl, false);
+    expected += 2 * ClassSize.estimateBase(AtomicLong.class, false);
+    expected += ClassSize.estimateBase(AtomicReference.class, false);
+    expected += ClassSize.estimateBase(CellSet.class, false);
+    expected += ClassSize.estimateBase(ReentrantReadWriteLock.class, false);
+    expected += ClassSize.estimateBase(NonSyncTimeRangeTracker.class, false);
+    if (expected != actual) {
+      ClassSize.estimateBase(cl, true);
+      ClassSize.estimateBase(AtomicLong.class, true);
+      ClassSize.estimateBase(AtomicLong.class, true);
+      ClassSize.estimateBase(AtomicReference.class, true);
+      ClassSize.estimateBase(CellSet.class, true);
+      ClassSize.estimateBase(ReentrantReadWriteLock.class,true);
+      ClassSize.estimateBase(NonSyncTimeRangeTracker.class, true);
+      assertEquals(expected, actual);
+    }
+
+    cl = CSLMImmutableSegment.class;
+    actual = CSLMImmutableSegment.DEEP_OVERHEAD_CSLM;
+    expected = ClassSize.estimateBase(cl, false);
+    expected += 2 * ClassSize.estimateBase(AtomicLong.class, false);
+    expected += ClassSize.estimateBase(AtomicReference.class, false);
+    expected += ClassSize.estimateBase(CellSet.class, false);
+    expected += ClassSize.estimateBase(ReentrantReadWriteLock.class, false);
+    expected += ClassSize.estimateBase(NonSyncTimeRangeTracker.class, false);
+    expected += ClassSize.estimateBase(ConcurrentSkipListMap.class, false);
+    if (expected != actual) {
+      ClassSize.estimateBase(cl, true);
+      ClassSize.estimateBase(AtomicLong.class, true);
+      ClassSize.estimateBase(AtomicLong.class, true);
+      ClassSize.estimateBase(AtomicReference.class, true);
+      ClassSize.estimateBase(CellSet.class, true);
+      ClassSize.estimateBase(ReentrantReadWriteLock.class,true);
+      ClassSize.estimateBase(NonSyncTimeRangeTracker.class, true);
       ClassSize.estimateBase(ConcurrentSkipListMap.class, true);
-      ClassSize.estimateBase(TimeRangeTracker.class, true);
-      ClassSize.estimateBase(TimeRangeTracker.class, true);
+      assertEquals(expected, actual);
+    }
+    cl = CellArrayImmutableSegment.class;
+    actual = CellArrayImmutableSegment.DEEP_OVERHEAD_CAM;
+    expected = ClassSize.estimateBase(cl, false);
+    expected += 2 * ClassSize.estimateBase(AtomicLong.class, false);
+    expected += ClassSize.estimateBase(AtomicReference.class, false);
+    expected += ClassSize.estimateBase(CellSet.class, false);
+    expected += ClassSize.estimateBase(ReentrantReadWriteLock.class, false);
+    expected += ClassSize.estimateBase(NonSyncTimeRangeTracker.class, false);
+    expected += ClassSize.estimateBase(CellArrayMap.class, false);
+    if (expected != actual) {
+      ClassSize.estimateBase(cl, true);
+      ClassSize.estimateBase(AtomicLong.class, true);
+      ClassSize.estimateBase(AtomicLong.class, true);
+      ClassSize.estimateBase(AtomicReference.class, true);
+      ClassSize.estimateBase(CellSet.class, true);
+      ClassSize.estimateBase(ReentrantReadWriteLock.class,true);
+      ClassSize.estimateBase(NonSyncTimeRangeTracker.class, true);
+      ClassSize.estimateBase(CellArrayMap.class, true);
       assertEquals(expected, actual);
     }
 
@@ -341,6 +494,14 @@ public class TestHeapSize  {
       assertEquals(expected, actual);
     }
 
+    cl = StoreHotnessProtector.class;
+    actual = StoreHotnessProtector.FIXED_SIZE;
+    expected = ClassSize.estimateBase(cl, false);
+    if (expected != actual) {
+      ClassSize.estimateBase(cl, true);
+      assertEquals(expected, actual);
+    }
+
     // Block cache key overhead. Only tests fixed overhead as estimating heap
     // size of strings is hard.
     cl = BlockCacheKey.class;
@@ -359,6 +520,28 @@ public class TestHeapSize  {
     // any of these classes are modified without updating overhead sizes.
   }
 
+  @Test
+  public void testHFileBlockSize() throws IOException {
+    long expected;
+    long actual;
+
+    actual = HFileContext.FIXED_OVERHEAD;
+    expected = ClassSize.estimateBase(HFileContext.class, false);
+    assertEquals(expected, actual);
+
+    actual = HFileBlock.FIXED_OVERHEAD;
+    expected = ClassSize.estimateBase(HFileBlock.class, false);
+    assertEquals(expected, actual);
+
+    actual = ExclusiveMemHFileBlock.FIXED_OVERHEAD;
+    expected = ClassSize.estimateBase(ExclusiveMemHFileBlock.class, false);
+    assertEquals(expected, actual);
+
+    actual = SharedMemHFileBlock.FIXED_OVERHEAD;
+    expected = ClassSize.estimateBase(SharedMemHFileBlock.class, false);
+    assertEquals(expected, actual);
+  }
+
   @Test
   public void testMutations(){
     Class<?> cl;
@@ -375,7 +558,7 @@ public class TestHeapSize  {
 
     byte[] row = new byte[] { 0 };
     cl = Put.class;
-    actual = new Put(row).MUTATION_OVERHEAD + ClassSize.align(ClassSize.ARRAY);
+    actual = Mutation.MUTATION_OVERHEAD + ClassSize.align(ClassSize.ARRAY);
     expected = ClassSize.estimateBase(cl, false);
     //The actual TreeMap is not included in the above calculation
     expected += ClassSize.align(ClassSize.TREEMAP);
@@ -386,7 +569,7 @@ public class TestHeapSize  {
     }
 
     cl = Delete.class;
-    actual = new Delete(row).MUTATION_OVERHEAD + ClassSize.align(ClassSize.ARRAY);
+    actual = Mutation.MUTATION_OVERHEAD + ClassSize.align(ClassSize.ARRAY);
     expected  = ClassSize.estimateBase(cl, false);
     //The actual TreeMap is not included in the above calculation
     expected += ClassSize.align(ClassSize.TREEMAP);
@@ -414,8 +597,25 @@ public class TestHeapSize  {
     } else {
       assertTrue(ClassSize.OBJECT == 12 || ClassSize.OBJECT == 16); // depending on CompressedOops
     }
-    assertEquals(ClassSize.OBJECT + 4, ClassSize.ARRAY);
+    if (ClassSize.useUnsafeLayout()) {
+      assertEquals(ClassSize.ARRAY, ClassSize.OBJECT + 4);
+    } else {
+      assertEquals(ClassSize.ARRAY, ClassSize.OBJECT + 8);
+    }
   }
 
-}
+  @Test
+  public void testAutoCalcFixedOverHead() {
+    Class[] classList = new Class[] { HFileContext.class, HRegion.class, BlockCacheKey.class,
+      HFileBlock.class, HStore.class, LruBlockCache.class, StoreContext.class };
+    for (Class cl : classList) {
+      // do estimate in advance to ensure class is loaded
+      ClassSize.estimateBase(cl, false);
 
+      long startTime = System.currentTimeMillis();
+      ClassSize.estimateBase(cl, false);
+      long endTime = System.currentTimeMillis();
+      assertTrue(endTime - startTime < 5);
+    }
+  }
+}
