@@ -16,11 +16,11 @@
  * limitations under the License.
  */
 package org.apache.hadoop.hbase.regionserver;
+
 import static org.apache.hadoop.hbase.HBaseTestingUtility.fam1;
 import static org.apache.hadoop.hbase.HBaseTestingUtility.fam2;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertNotNull;
-import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 
@@ -28,18 +28,18 @@ import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
+import java.util.Objects;
 import java.util.Random;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.atomic.AtomicLong;
-
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.hbase.Cell;
 import org.apache.hadoop.hbase.CellUtil;
+import org.apache.hadoop.hbase.CompareOperator;
+import org.apache.hadoop.hbase.HBaseClassTestRule;
 import org.apache.hadoop.hbase.HBaseTestingUtility;
 import org.apache.hadoop.hbase.HColumnDescriptor;
 import org.apache.hadoop.hbase.HConstants;
@@ -54,32 +54,44 @@ import org.apache.hadoop.hbase.client.Delete;
 import org.apache.hadoop.hbase.client.Durability;
 import org.apache.hadoop.hbase.client.Get;
 import org.apache.hadoop.hbase.client.Increment;
+import org.apache.hadoop.hbase.client.IsolationLevel;
 import org.apache.hadoop.hbase.client.Mutation;
 import org.apache.hadoop.hbase.client.Put;
+import org.apache.hadoop.hbase.client.RegionInfo;
 import org.apache.hadoop.hbase.client.Result;
 import org.apache.hadoop.hbase.client.RowMutations;
 import org.apache.hadoop.hbase.client.Scan;
+import org.apache.hadoop.hbase.client.TableDescriptor;
 import org.apache.hadoop.hbase.filter.BinaryComparator;
-import org.apache.hadoop.hbase.filter.CompareFilter.CompareOp;
 import org.apache.hadoop.hbase.io.HeapSize;
 import org.apache.hadoop.hbase.io.hfile.BlockCache;
-import org.apache.hadoop.hbase.testclassification.MediumTests;
+import org.apache.hadoop.hbase.io.hfile.CacheConfig;
+import org.apache.hadoop.hbase.testclassification.LargeTests;
+import org.apache.hadoop.hbase.testclassification.VerySlowRegionServerTests;
 import org.apache.hadoop.hbase.util.Bytes;
 import org.apache.hadoop.hbase.wal.WAL;
 import org.junit.After;
 import org.junit.Before;
+import org.junit.ClassRule;
 import org.junit.Rule;
 import org.junit.Test;
 import org.junit.experimental.categories.Category;
 import org.junit.rules.TestName;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 /**
  * Testing of HRegion.incrementColumnValue, HRegion.increment,
  * and HRegion.append
  */
-@Category(MediumTests.class) // Starts 100 threads
+@Category({VerySlowRegionServerTests.class, LargeTests.class}) // Starts 100 threads
 public class TestAtomicOperation {
-  private static final Log LOG = LogFactory.getLog(TestAtomicOperation.class);
+
+  @ClassRule
+  public static final HBaseClassTestRule CLASS_RULE =
+      HBaseClassTestRule.forClass(TestAtomicOperation.class);
+
+  private static final Logger LOG = LoggerFactory.getLogger(TestAtomicOperation.class);
   @Rule public TestName name = new TestName();
 
   HRegion region = null;
@@ -103,14 +115,17 @@ public class TestAtomicOperation {
   @After
   public void teardown() throws IOException {
     if (region != null) {
-      BlockCache bc = region.getStores().get(0).getCacheConfig().getBlockCache();
-      ((HRegion)region).close();
-      WAL wal = ((HRegion)region).getWAL();
-      if (wal != null) wal.close();
-      if (bc != null) bc.shutdown();
+      CacheConfig cacheConfig = region.getStores().get(0).getCacheConfig();
+      region.close();
+      WAL wal = region.getWAL();
+      if (wal != null) {
+        wal.close();
+      }
+      cacheConfig.getBlockCache().ifPresent(BlockCache::shutdown);
       region = null;
     }
   }
+
   //////////////////////////////////////////////////////////////////////////////
   // New tests that doesn't spin up a mini cluster but rather just test the
   // individual code pieces in the HRegion.
@@ -129,12 +144,12 @@ public class TestAtomicOperation {
     String v2 = " is... 42.";
     Append a = new Append(row);
     a.setReturnResults(false);
-    a.add(fam1, qual1, Bytes.toBytes(v1));
-    a.add(fam1, qual2, Bytes.toBytes(v2));
-    assertNull(region.append(a, HConstants.NO_NONCE, HConstants.NO_NONCE));
+    a.addColumn(fam1, qual1, Bytes.toBytes(v1));
+    a.addColumn(fam1, qual2, Bytes.toBytes(v2));
+    assertTrue(region.append(a, HConstants.NO_NONCE, HConstants.NO_NONCE).isEmpty());
     a = new Append(row);
-    a.add(fam1, qual1, Bytes.toBytes(v2));
-    a.add(fam1, qual2, Bytes.toBytes(v1));
+    a.addColumn(fam1, qual1, Bytes.toBytes(v2));
+    a.addColumn(fam1, qual2, Bytes.toBytes(v1));
     Result result = region.append(a, HConstants.NO_NONCE, HConstants.NO_NONCE);
     assertEquals(0, Bytes.compareTo(Bytes.toBytes(v1+v2), result.getValue(fam1, qual1)));
     assertEquals(0, Bytes.compareTo(Bytes.toBytes(v2+v1), result.getValue(fam1, qual2)));
@@ -149,17 +164,19 @@ public class TestAtomicOperation {
 
     Append a = new Append(row);
     a.setReturnResults(false);
-    a.add(fam1, qual1, Bytes.toBytes(v1));
-    a.add(fam2, qual2, Bytes.toBytes(v2));
-    assertNull(region.append(a, HConstants.NO_NONCE, HConstants.NO_NONCE));
+    a.addColumn(fam1, qual1, Bytes.toBytes(v1));
+    a.addColumn(fam2, qual2, Bytes.toBytes(v2));
+    Result result = region.append(a, HConstants.NO_NONCE, HConstants.NO_NONCE);
+    assertTrue("Expected an empty result but result contains " + result.size() + " keys",
+      result.isEmpty());
 
     a = new Append(row);
-    a.add(fam2, qual2, Bytes.toBytes(v1));
-    a.add(fam1, qual1, Bytes.toBytes(v2));
-    a.add(fam3, qual3, Bytes.toBytes(v2));
-    a.add(fam1, qual2, Bytes.toBytes(v1));
+    a.addColumn(fam2, qual2, Bytes.toBytes(v1));
+    a.addColumn(fam1, qual1, Bytes.toBytes(v2));
+    a.addColumn(fam3, qual3, Bytes.toBytes(v2));
+    a.addColumn(fam1, qual2, Bytes.toBytes(v1));
 
-    Result result = region.append(a, HConstants.NO_NONCE, HConstants.NO_NONCE);
+    result = region.append(a, HConstants.NO_NONCE, HConstants.NO_NONCE);
 
     byte[] actualValue1 = result.getValue(fam1, qual1);
     byte[] actualValue2 = result.getValue(fam2, qual2);
@@ -181,8 +198,8 @@ public class TestAtomicOperation {
     initHRegion(tableName, name.getMethodName(), fam1);
     final String v1 = "Value";
     final Append a = new Append(row);
-    a.add(fam1, qual1, Bytes.toBytes(v1));
-    a.add(fam2, qual2, Bytes.toBytes(v1));
+    a.addColumn(fam1, qual1, Bytes.toBytes(v1));
+    a.addColumn(fam2, qual2, Bytes.toBytes(v1));
     Result result = null;
     try {
       result = region.append(a, HConstants.NO_NONCE, HConstants.NO_NONCE);
@@ -218,11 +235,13 @@ public class TestAtomicOperation {
    */
   @Test
   public void testIncrementMultiThreads() throws IOException {
+    boolean fast = true;
     LOG.info("Starting test testIncrementMultiThreads");
     // run a with mixed column families (1 and 3 versions)
     initHRegion(tableName, name.getMethodName(), new int[] {1,3}, fam1, fam2);
 
-    // Create 100 threads, each will increment by its own quantity
+    // Create 100 threads, each will increment by its own quantity. All 100 threads update the
+    // same row over two column families.
     int numThreads = 100;
     int incrementsPerThread = 1000;
     Incrementer[] all = new Incrementer[numThreads];
@@ -246,9 +265,9 @@ public class TestAtomicOperation {
         LOG.info("Ignored", e);
       }
     }
-    assertICV(row, fam1, qual1, expectedTotal);
-    assertICV(row, fam1, qual2, expectedTotal*2);
-    assertICV(row, fam2, qual3, expectedTotal*3);
+    assertICV(row, fam1, qual1, expectedTotal, fast);
+    assertICV(row, fam1, qual2, expectedTotal*2, fast);
+    assertICV(row, fam2, qual3, expectedTotal*3, fast);
     LOG.info("testIncrementMultiThreads successfully verified that total is " + expectedTotal);
   }
 
@@ -256,9 +275,11 @@ public class TestAtomicOperation {
   private void assertICV(byte [] row,
                          byte [] familiy,
                          byte[] qualifier,
-                         long amount) throws IOException {
+                         long amount,
+                         boolean fast) throws IOException {
     // run a get and see?
     Get get = new Get(row);
+    if (fast) get.setIsolationLevel(IsolationLevel.READ_UNCOMMITTED);
     get.addColumn(familiy, qualifier);
     Result result = region.get(get);
     assertEquals(1, result.size());
@@ -289,7 +310,8 @@ public class TestAtomicOperation {
   }
 
   /**
-   * A thread that makes a few increment calls
+   * A thread that makes increment calls always on the same row, this.row against two column
+   * families on this row.
    */
   public static class Incrementer extends Thread {
 
@@ -298,9 +320,8 @@ public class TestAtomicOperation {
     private final int amount;
 
 
-    public Incrementer(Region region,
-        int threadNumber, int amount, int numIncrements) {
-      super("incrementer." + threadNumber);
+    public Incrementer(Region region, int threadNumber, int amount, int numIncrements) {
+      super("Incrementer." + threadNumber);
       this.region = region;
       this.numIncrements = numIncrements;
       this.amount = amount;
@@ -316,19 +337,19 @@ public class TestAtomicOperation {
           inc.addColumn(fam1, qual2, amount*2);
           inc.addColumn(fam2, qual3, amount*3);
           inc.setDurability(Durability.ASYNC_WAL);
-          region.increment(inc, HConstants.NO_NONCE, HConstants.NO_NONCE);
-
-          // verify: Make sure we only see completed increments
-          Get g = new Get(row);
-          Result result = region.get(g);
+          Result result = region.increment(inc);
           if (result != null) {
-            assertTrue(result.getValue(fam1, qual1) != null);
-            assertTrue(result.getValue(fam1, qual2) != null);
             assertEquals(Bytes.toLong(result.getValue(fam1, qual1))*2,
               Bytes.toLong(result.getValue(fam1, qual2)));
             assertTrue(result.getValue(fam2, qual3) != null);
             assertEquals(Bytes.toLong(result.getValue(fam1, qual1))*3,
               Bytes.toLong(result.getValue(fam2, qual3)));
+            assertEquals(Bytes.toLong(result.getValue(fam1, qual1))*2,
+               Bytes.toLong(result.getValue(fam1, qual2)));
+            long fam1Increment = Bytes.toLong(result.getValue(fam1, qual1))*3;
+            long fam2Increment = Bytes.toLong(result.getValue(fam2, qual3));
+            assertEquals("fam1=" + fam1Increment + ", fam2=" + fam2Increment,
+              fam1Increment, fam2Increment);
           }
         } catch (IOException e) {
           e.printStackTrace();
@@ -357,18 +378,16 @@ public class TestAtomicOperation {
           for (int i=0; i<numOps; i++) {
             try {
               Append a = new Append(row);
-              a.add(fam1, qual1, val);
-              a.add(fam1, qual2, val);
-              a.add(fam2, qual3, val);
+              a.addColumn(fam1, qual1, val);
+              a.addColumn(fam1, qual2, val);
+              a.addColumn(fam2, qual3, val);
               a.setDurability(Durability.ASYNC_WAL);
               region.append(a, HConstants.NO_NONCE, HConstants.NO_NONCE);
 
               Get g = new Get(row);
               Result result = region.get(g);
-              assertEquals(result.getValue(fam1, qual1).length,
-                  result.getValue(fam1, qual2).length);
-              assertEquals(result.getValue(fam1, qual1).length,
-                  result.getValue(fam2, qual3).length);
+              assertEquals(result.getValue(fam1, qual1).length, result.getValue(fam1, qual2).length);
+              assertEquals(result.getValue(fam1, qual1).length, result.getValue(fam2, qual3).length);
             } catch (IOException e) {
               e.printStackTrace();
               failures.incrementAndGet();
@@ -394,9 +413,9 @@ public class TestAtomicOperation {
     assertEquals(0, failures.get());
     Get g = new Get(row);
     Result result = region.get(g);
-    assertEquals(result.getValue(fam1, qual1).length, 10000);
-    assertEquals(result.getValue(fam1, qual2).length, 10000);
-    assertEquals(result.getValue(fam2, qual3).length, 10000);
+    assertEquals(10000, result.getValue(fam1, qual1).length);
+    assertEquals(10000, result.getValue(fam1, qual2).length);
+    assertEquals(10000, result.getValue(fam2, qual3).length);
   }
   /**
    * Test multi-threaded row mutations.
@@ -436,20 +455,20 @@ public class TestAtomicOperation {
               RowMutations rm = new RowMutations(row);
               if (op) {
                 Put p = new Put(row, ts);
-                p.add(fam1, qual1, value1);
+                p.addColumn(fam1, qual1, value1);
                 p.setDurability(Durability.ASYNC_WAL);
                 rm.add(p);
                 Delete d = new Delete(row);
-                d.deleteColumns(fam1, qual2, ts);
+                d.addColumns(fam1, qual2, ts);
                 d.setDurability(Durability.ASYNC_WAL);
                 rm.add(d);
               } else {
                 Delete d = new Delete(row);
-                d.deleteColumns(fam1, qual1, ts);
+                d.addColumns(fam1, qual1, ts);
                 d.setDurability(Durability.ASYNC_WAL);
                 rm.add(d);
                 Put p = new Put(row, ts);
-                p.add(fam1, qual2, value2);
+                p.addColumn(fam1, qual2, value2);
                 p.setDurability(Durability.ASYNC_WAL);
                 rm.add(p);
               }
@@ -459,7 +478,7 @@ public class TestAtomicOperation {
               Get g = new Get(row);
               Result r = region.get(g);
               if (r.size() != 1) {
-                LOG.debug(r);
+                LOG.debug(Objects.toString(r));
                 failures.incrementAndGet();
                 fail();
               }
@@ -526,24 +545,24 @@ public class TestAtomicOperation {
                 }
               }
               long ts = timeStamps.incrementAndGet();
-              List<Mutation> mrm = new ArrayList<Mutation>();
+              List<Mutation> mrm = new ArrayList<>();
               if (op) {
                 Put p = new Put(row2, ts);
-                p.add(fam1, qual1, value1);
+                p.addColumn(fam1, qual1, value1);
                 p.setDurability(Durability.ASYNC_WAL);
                 mrm.add(p);
                 Delete d = new Delete(row);
-                d.deleteColumns(fam1, qual1, ts);
+                d.addColumns(fam1, qual1, ts);
                 d.setDurability(Durability.ASYNC_WAL);
                 mrm.add(d);
               } else {
                 Delete d = new Delete(row2);
-                d.deleteColumns(fam1, qual1, ts);
+                d.addColumns(fam1, qual1, ts);
                 d.setDurability(Durability.ASYNC_WAL);
                 mrm.add(d);
                 Put p = new Put(row, ts);
                 p.setDurability(Durability.ASYNC_WAL);
-                p.add(fam1, qual1, value2);
+                p.addColumn(fam1, qual1, value2);
                 mrm.add(p);
               }
               region.mutateRowsWithLocks(mrm, rowsToLock, HConstants.NO_NONCE, HConstants.NO_NONCE);
@@ -551,12 +570,12 @@ public class TestAtomicOperation {
               // check: should always see exactly one column
               Scan s = new Scan(row);
               RegionScanner rs = region.getScanner(s);
-              List<Cell> r = new ArrayList<Cell>();
+              List<Cell> r = new ArrayList<>();
               while (rs.next(r))
                 ;
               rs.close();
               if (r.size() != 1) {
-                LOG.debug(r);
+                LOG.debug(Objects.toString(r));
                 failures.incrementAndGet();
                 fail();
               }
@@ -621,19 +640,17 @@ public class TestAtomicOperation {
    */
   @Test
   public void testPutAndCheckAndPutInParallel() throws Exception {
-
-    final String tableName = "testPutAndCheckAndPut";
     Configuration conf = TEST_UTIL.getConfiguration();
     conf.setClass(HConstants.REGION_IMPL, MockHRegion.class, HeapSize.class);
-    final MockHRegion region = (MockHRegion) TEST_UTIL.createLocalHRegion(Bytes.toBytes(tableName),
-        null, null, tableName, conf, false, Durability.SYNC_WAL, null, Bytes.toBytes(family));
-
+    HTableDescriptor htd = new HTableDescriptor(TableName.valueOf(name.getMethodName()))
+        .addFamily(new HColumnDescriptor(family));
+    this.region = TEST_UTIL.createLocalHRegion(htd, null, null);
     Put[] puts = new Put[1];
     Put put = new Put(Bytes.toBytes("r1"));
-    put.add(Bytes.toBytes(family), Bytes.toBytes("q1"), Bytes.toBytes("10"));
+    put.addColumn(Bytes.toBytes(family), Bytes.toBytes("q1"), Bytes.toBytes("10"));
     puts[0] = put;
 
-    region.batchMutate(puts, HConstants.NO_NONCE, HConstants.NO_NONCE);
+    region.batchMutate(puts);
     MultithreadedTestUtil.TestContext ctx =
       new MultithreadedTestUtil.TestContext(conf);
     ctx.addThread(new PutThread(ctx, region));
@@ -645,7 +662,7 @@ public class TestAtomicOperation {
     ctx.stop();
     Scan s = new Scan();
     RegionScanner scanner = region.getScanner(s);
-    List<Cell> results = new ArrayList<Cell>();
+    List<Cell> results = new ArrayList<>();
     ScannerContext scannerContext = ScannerContext.newBuilder().setBatchLimit(2).build();
     scanner.next(results, scannerContext);
     for (Cell keyValue : results) {
@@ -654,40 +671,42 @@ public class TestAtomicOperation {
   }
 
   private class PutThread extends TestThread {
-    private HRegion region;
-    PutThread(TestContext ctx, HRegion region) {
+    private Region region;
+    PutThread(TestContext ctx, Region region) {
       super(ctx);
       this.region = region;
     }
 
+    @Override
     public void doWork() throws Exception {
       Put[] puts = new Put[1];
       Put put = new Put(Bytes.toBytes("r1"));
-      put.add(Bytes.toBytes(family), Bytes.toBytes("q1"), Bytes.toBytes("50"));
+      put.addColumn(Bytes.toBytes(family), Bytes.toBytes("q1"), Bytes.toBytes("50"));
       puts[0] = put;
       testStep = TestStep.PUT_STARTED;
-      region.batchMutate(puts, HConstants.NO_NONCE, HConstants.NO_NONCE);
+      region.batchMutate(puts);
     }
   }
 
   private class CheckAndPutThread extends TestThread {
-    private HRegion region;
-    CheckAndPutThread(TestContext ctx, HRegion region) {
+    private Region region;
+    CheckAndPutThread(TestContext ctx, Region region) {
       super(ctx);
       this.region = region;
    }
 
+    @Override
     public void doWork() throws Exception {
       Put[] puts = new Put[1];
       Put put = new Put(Bytes.toBytes("r1"));
-      put.add(Bytes.toBytes(family), Bytes.toBytes("q1"), Bytes.toBytes("11"));
+      put.addColumn(Bytes.toBytes(family), Bytes.toBytes("q1"), Bytes.toBytes("11"));
       puts[0] = put;
       while (testStep != TestStep.PUT_COMPLETED) {
         Thread.sleep(100);
       }
       testStep = TestStep.CHECKANDPUT_STARTED;
       region.checkAndMutate(Bytes.toBytes("r1"), Bytes.toBytes(family), Bytes.toBytes("q1"),
-        CompareOp.EQUAL, new BinaryComparator(Bytes.toBytes("10")), put, true);
+        CompareOperator.EQUAL, new BinaryComparator(Bytes.toBytes("10")), put);
       testStep = TestStep.CHECKANDPUT_COMPLETED;
     }
   }
@@ -695,17 +714,17 @@ public class TestAtomicOperation {
   public static class MockHRegion extends HRegion {
 
     public MockHRegion(Path tableDir, WAL log, FileSystem fs, Configuration conf,
-        final HRegionInfo regionInfo, final HTableDescriptor htd, RegionServerServices rsServices) {
+        final RegionInfo regionInfo, final TableDescriptor htd, RegionServerServices rsServices) {
       super(tableDir, log, fs, conf, regionInfo, htd, rsServices);
     }
 
     @Override
-    public RowLock getRowLockInternal(final byte[] row, boolean readLock, boolean waitForLock,
-        final RowLock prevRowLock) throws IOException {
+    public RowLock getRowLockInternal(final byte[] row, boolean readLock,
+        final RowLock prevRowlock) throws IOException {
       if (testStep == TestStep.CHECKANDPUT_STARTED) {
         latch.countDown();
       }
-      return new WrappedRowLock(super.getRowLockInternal(row, readLock, waitForLock, prevRowLock));
+      return new WrappedRowLock(super.getRowLockInternal(row, readLock, null));
     }
 
     public class WrappedRowLock implements RowLock {
