@@ -21,7 +21,8 @@ package org.apache.hadoop.hbase.thrift2;
 import static org.apache.hadoop.hbase.thrift.Constants.THRIFT_READONLY_ENABLED;
 import static org.apache.hadoop.hbase.thrift.Constants.THRIFT_READONLY_ENABLED_DEFAULT;
 import static org.apache.hadoop.hbase.thrift2.ThriftUtilities.appendFromThrift;
-import static org.apache.hadoop.hbase.thrift2.ThriftUtilities.columnDescriptorFromThrift;
+import static org.apache.hadoop.hbase.thrift2.ThriftUtilities.columnFamilyDescriptorFromThrift;
+import static org.apache.hadoop.hbase.thrift2.ThriftUtilities.compareOpFromThrift;
 import static org.apache.hadoop.hbase.thrift2.ThriftUtilities.deleteFromThrift;
 import static org.apache.hadoop.hbase.thrift2.ThriftUtilities.deletesFromThrift;
 import static org.apache.hadoop.hbase.thrift2.ThriftUtilities.getFromThrift;
@@ -32,8 +33,6 @@ import static org.apache.hadoop.hbase.thrift2.ThriftUtilities.namespaceDescripto
 import static org.apache.hadoop.hbase.thrift2.ThriftUtilities.namespaceDescriptorsFromHBase;
 import static org.apache.hadoop.hbase.thrift2.ThriftUtilities.putFromThrift;
 import static org.apache.hadoop.hbase.thrift2.ThriftUtilities.putsFromThrift;
-import static org.apache.hadoop.hbase.thrift2.ThriftUtilities.regionLocationFromHBase;
-import static org.apache.hadoop.hbase.thrift2.ThriftUtilities.regionLocationsFromHBase;
 import static org.apache.hadoop.hbase.thrift2.ThriftUtilities.resultFromHBase;
 import static org.apache.hadoop.hbase.thrift2.ThriftUtilities.resultsFromHBase;
 import static org.apache.hadoop.hbase.thrift2.ThriftUtilities.rowMutationsFromThrift;
@@ -44,13 +43,11 @@ import static org.apache.hadoop.hbase.thrift2.ThriftUtilities.tableDescriptorFro
 import static org.apache.hadoop.hbase.thrift2.ThriftUtilities.tableDescriptorsFromHBase;
 import static org.apache.hadoop.hbase.thrift2.ThriftUtilities.tableNameFromThrift;
 import static org.apache.hadoop.hbase.thrift2.ThriftUtilities.tableNamesFromHBase;
-import static org.apache.hadoop.hbase.thrift2.ThriftUtilities.tableNamesFromThrift;
 import static org.apache.thrift.TBaseHelper.byteBufferToByteArray;
 
 import java.io.IOException;
 import java.nio.ByteBuffer;
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.Collections;
 import java.util.List;
 import java.util.Map;
@@ -59,21 +56,19 @@ import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.regex.Pattern;
 
-import org.apache.commons.lang.NotImplementedException;
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.hbase.DoNotRetryIOException;
-import org.apache.hadoop.hbase.HColumnDescriptor;
 import org.apache.hadoop.hbase.HRegionLocation;
-import org.apache.hadoop.hbase.HTableDescriptor;
 import org.apache.hadoop.hbase.NamespaceDescriptor;
+import org.apache.hadoop.hbase.ServerName;
 import org.apache.hadoop.hbase.TableName;
-import org.apache.hadoop.hbase.classification.InterfaceAudience;
+import org.apache.hadoop.hbase.client.ColumnFamilyDescriptor;
 import org.apache.hadoop.hbase.client.RegionLocator;
 import org.apache.hadoop.hbase.client.ResultScanner;
+import org.apache.hadoop.hbase.client.LogQueryFilter;
+import org.apache.hadoop.hbase.client.OnlineLogRecord;
 import org.apache.hadoop.hbase.client.Table;
-import org.apache.hadoop.hbase.filter.CompareFilter;
+import org.apache.hadoop.hbase.client.TableDescriptor;
 import org.apache.hadoop.hbase.security.UserProvider;
 import org.apache.hadoop.hbase.thrift.HBaseServiceHandler;
 import org.apache.hadoop.hbase.thrift2.generated.TAppend;
@@ -99,6 +94,9 @@ import org.apache.hadoop.hbase.thrift2.generated.TTableName;
 import org.apache.hadoop.hbase.thrift2.generated.TThriftServerType;
 import org.apache.hadoop.hbase.util.Bytes;
 import org.apache.thrift.TException;
+import org.apache.yetus.audience.InterfaceAudience;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 /**
  * This class is a glue object that connects Thrift RPC calls to the HBase client API primarily
@@ -109,7 +107,7 @@ import org.apache.thrift.TException;
 public class ThriftHBaseServiceHandler extends HBaseServiceHandler implements THBaseService.Iface {
 
   // TODO: Size of pool configuraple
-  private static final Log LOG = LogFactory.getLog(ThriftHBaseServiceHandler.class);
+  private static final Logger LOG = LoggerFactory.getLogger(ThriftHBaseServiceHandler.class);
 
   // nextScannerId and scannerMap are used to manage scanner state
   // TODO: Cleanup thread for Scanners, Scanner id wrap
@@ -188,8 +186,8 @@ public class ThriftHBaseServiceHandler extends HBaseServiceHandler implements TH
 
   private TIOError getTIOError(IOException e) {
     TIOError err = new TIOErrorWithCause(e);
-    err.setMessage(e.getMessage());
     err.setCanRetry(!(e instanceof DoNotRetryIOException));
+    err.setMessage(e.getMessage());
     return err;
   }
 
@@ -292,11 +290,15 @@ public class ThriftHBaseServiceHandler extends HBaseServiceHandler implements TH
   public boolean checkAndPut(ByteBuffer table, ByteBuffer row, ByteBuffer family,
       ByteBuffer qualifier, ByteBuffer value, TPut put) throws TIOError, TException {
     checkReadOnlyMode();
-
     Table htable = getTable(table);
     try {
-      return htable.checkAndPut(byteBufferToByteArray(row), byteBufferToByteArray(family),
-          byteBufferToByteArray(qualifier), byteBufferToByteArray(value), putFromThrift(put));
+      Table.CheckAndMutateBuilder builder = htable.checkAndMutate(byteBufferToByteArray(row),
+          byteBufferToByteArray(family)).qualifier(byteBufferToByteArray(qualifier));
+      if (value == null) {
+        return builder.ifNotExists().thenPut(putFromThrift(put));
+      } else {
+        return builder.ifEquals(byteBufferToByteArray(value)).thenPut(putFromThrift(put));
+      }
     } catch (IOException e) {
       throw getTIOError(e);
     } finally {
@@ -344,40 +346,17 @@ public class ThriftHBaseServiceHandler extends HBaseServiceHandler implements TH
     }
     return Collections.emptyList();
   }
-
+  
   @Override
   public boolean checkAndMutate(ByteBuffer table, ByteBuffer row, ByteBuffer family,
       ByteBuffer qualifier, TCompareOp compareOp, ByteBuffer value, TRowMutations rowMutations)
-      throws TIOError, TException {
+          throws TIOError, TException {
     checkReadOnlyMode();
     try (final Table htable = getTable(table)) {
-      return htable.checkAndMutate(byteBufferToByteArray(row), byteBufferToByteArray(family),
-          byteBufferToByteArray(qualifier), CompareFilter.CompareOp.EQUAL,
-          byteBufferToByteArray(value), rowMutationsFromThrift(rowMutations));
-    } catch (IOException e) {
-      throw getTIOError(e);
-    }
-  }
-
-  @Override
-  public TTableDescriptor getTableDescriptor(TTableName table)
-      throws TIOError, TException {
-    try {
-      TableName tableName = tableNameFromThrift(table);
-      HTableDescriptor tableDescriptor = connectionCache.getAdmin().getTableDescriptor(tableName);
-      return tableDescriptorFromHBase(tableDescriptor);
-    } catch (IOException e) {
-      throw getTIOError(e);
-    }
-  }
-
-  @Override
-  public List<TTableDescriptor> getTableDescriptors(List<TTableName> tables)
-      throws TIOError, TException {
-    try {
-      List<TableName> tableNames = tableNamesFromThrift(tables);
-      return tableDescriptorsFromHBase(
-        Arrays.asList(connectionCache.getAdmin().getTableDescriptorsByTableName(tableNames)));
+      return htable.checkAndMutate(byteBufferToByteArray(row), byteBufferToByteArray(family))
+          .qualifier(byteBufferToByteArray(qualifier))
+          .ifMatches(compareOpFromThrift(compareOp), byteBufferToByteArray(value))
+          .thenMutate(rowMutationsFromThrift(rowMutations));
     } catch (IOException e) {
       throw getTIOError(e);
     }
@@ -389,9 +368,15 @@ public class ThriftHBaseServiceHandler extends HBaseServiceHandler implements TH
     checkReadOnlyMode();
     Table htable = getTable(table);
     try {
-      return htable.checkAndDelete(byteBufferToByteArray(row),
-          byteBufferToByteArray(family), byteBufferToByteArray(qualifier),
-          byteBufferToByteArray(value), deleteFromThrift(deleteSingle));
+      Table.CheckAndMutateBuilder mutateBuilder =
+          htable.checkAndMutate(byteBufferToByteArray(row), byteBufferToByteArray(family))
+              .qualifier(byteBufferToByteArray(qualifier));
+      if (value == null) {
+        return mutateBuilder.ifNotExists().thenDelete(deleteFromThrift(deleteSingle));
+      } else {
+        return mutateBuilder.ifEquals(byteBufferToByteArray(value))
+            .thenDelete(deleteFromThrift(deleteSingle));
+      }
     } catch (IOException e) {
       throw getTIOError(e);
     } finally {
@@ -476,6 +461,8 @@ public class ThriftHBaseServiceHandler extends HBaseServiceHandler implements TH
     return results;
   }
 
+
+
   @Override
   public void closeScanner(int scannerId) throws TIOError, TIllegalArgument, TException {
     LOG.debug("scannerClose: id=" + scannerId);
@@ -510,7 +497,8 @@ public class ThriftHBaseServiceHandler extends HBaseServiceHandler implements TH
     RegionLocator locator = null;
     try {
       locator = getLocator(table);
-      return regionLocationsFromHBase(locator.getAllRegionLocations());
+      return ThriftUtilities.regionLocationsFromHBase(locator.getAllRegionLocations());
+
     } catch (IOException e) {
       throw getTIOError(e);
     } finally {
@@ -533,7 +521,8 @@ public class ThriftHBaseServiceHandler extends HBaseServiceHandler implements TH
       locator = getLocator(table);
       byte[] rowBytes = byteBufferToByteArray(row);
       HRegionLocation hrl = locator.getRegionLocation(rowBytes, reload);
-      return regionLocationFromHBase(hrl);
+      return ThriftUtilities.regionLocationFromHBase(hrl);
+
     } catch (IOException e) {
       throw getTIOError(e);
     } finally {
@@ -557,6 +546,30 @@ public class ThriftHBaseServiceHandler extends HBaseServiceHandler implements TH
     return isReadOnly;
   }
 
+  @Override
+  public TTableDescriptor getTableDescriptor(TTableName table) throws TIOError, TException {
+    try {
+      TableName tableName = ThriftUtilities.tableNameFromThrift(table);
+      TableDescriptor tableDescriptor = connectionCache.getAdmin().getDescriptor(tableName);
+      return tableDescriptorFromHBase(tableDescriptor);
+    } catch (IOException e) {
+      throw getTIOError(e);
+    }
+  }
+
+  @Override
+  public List<TTableDescriptor> getTableDescriptors(List<TTableName> tables)
+      throws TIOError, TException {
+    try {
+      List<TableName> tableNames = ThriftUtilities.tableNamesFromThrift(tables);
+      List<TableDescriptor> tableDescriptors = connectionCache.getAdmin()
+          .listTableDescriptors(tableNames);
+      return tableDescriptorsFromHBase(tableDescriptors);
+    } catch (IOException e) {
+      throw getTIOError(e);
+    }
+  }
+
   @Override
   public boolean tableExists(TTableName tTableName) throws TIOError, TException {
     try {
@@ -572,8 +585,8 @@ public class ThriftHBaseServiceHandler extends HBaseServiceHandler implements TH
       throws TIOError, TException {
     try {
       Pattern pattern = (regex == null ? null : Pattern.compile(regex));
-      List<HTableDescriptor> tableDescriptors =
-        Arrays.asList(connectionCache.getAdmin().listTables(pattern));
+      List<TableDescriptor> tableDescriptors = connectionCache.getAdmin()
+          .listTableDescriptors(pattern, includeSysTables);
       return tableDescriptorsFromHBase(tableDescriptors);
     } catch (IOException e) {
       throw getTIOError(e);
@@ -584,8 +597,9 @@ public class ThriftHBaseServiceHandler extends HBaseServiceHandler implements TH
   public List<TTableDescriptor> getTableDescriptorsByNamespace(String name)
       throws TIOError, TException {
     try {
-      HTableDescriptor[] tables = connectionCache.getAdmin().listTableDescriptorsByNamespace(name);
-      return tableDescriptorsFromHBase(Arrays.asList(tables));
+      List<TableDescriptor> descriptors = connectionCache.getAdmin()
+          .listTableDescriptorsByNamespace(Bytes.toBytes(name));
+      return tableDescriptorsFromHBase(descriptors);
     } catch (IOException e) {
       throw getTIOError(e);
     }
@@ -618,7 +632,7 @@ public class ThriftHBaseServiceHandler extends HBaseServiceHandler implements TH
   public void createTable(TTableDescriptor desc, List<ByteBuffer> splitKeys)
       throws TIOError, TException {
     try {
-      HTableDescriptor descriptor = tableDescriptorFromThrift(desc);
+      TableDescriptor descriptor = tableDescriptorFromThrift(desc);
       byte[][] split = splitKeyFromThrift(splitKeys);
       connectionCache.getAdmin().createTable(descriptor, split);
     } catch (IOException e) {
@@ -714,8 +728,8 @@ public class ThriftHBaseServiceHandler extends HBaseServiceHandler implements TH
       throws TIOError, TException {
     try {
       TableName table = tableNameFromThrift(tableName);
-      HColumnDescriptor columnFamilyDescriptor = columnDescriptorFromThrift(column);
-      connectionCache.getAdmin().addColumn(table, columnFamilyDescriptor);
+      ColumnFamilyDescriptor columnFamilyDescriptor = columnFamilyDescriptorFromThrift(column);
+      connectionCache.getAdmin().addColumnFamily(table, columnFamilyDescriptor);
     } catch (IOException e) {
       throw getTIOError(e);
     }
@@ -726,7 +740,7 @@ public class ThriftHBaseServiceHandler extends HBaseServiceHandler implements TH
       throws TIOError, TException {
     try {
       TableName table = tableNameFromThrift(tableName);
-      connectionCache.getAdmin().deleteColumn(table, column.array());
+      connectionCache.getAdmin().deleteColumnFamily(table, column.array());
     } catch (IOException e) {
       throw getTIOError(e);
     }
@@ -737,8 +751,8 @@ public class ThriftHBaseServiceHandler extends HBaseServiceHandler implements TH
       throws TIOError, TException {
     try {
       TableName table = tableNameFromThrift(tableName);
-      HColumnDescriptor columnFamilyDescriptor = columnDescriptorFromThrift(column);
-      connectionCache.getAdmin().modifyColumn(table, columnFamilyDescriptor);
+      ColumnFamilyDescriptor columnFamilyDescriptor = columnFamilyDescriptorFromThrift(column);
+      connectionCache.getAdmin().modifyColumnFamily(table, columnFamilyDescriptor);
     } catch (IOException e) {
       throw getTIOError(e);
     }
@@ -747,16 +761,15 @@ public class ThriftHBaseServiceHandler extends HBaseServiceHandler implements TH
   @Override
   public void modifyTable(TTableDescriptor desc) throws TIOError, TException {
     try {
-      HTableDescriptor descriptor = tableDescriptorFromThrift(desc);
-      connectionCache.getAdmin().modifyTable(descriptor.getTableName(), descriptor);
+      TableDescriptor descriptor = tableDescriptorFromThrift(desc);
+      connectionCache.getAdmin().modifyTable(descriptor);
     } catch (IOException e) {
       throw getTIOError(e);
     }
   }
 
   @Override
-  public void createNamespace(TNamespaceDescriptor namespaceDesc)
-      throws TIOError, TException {
+  public void createNamespace(TNamespaceDescriptor namespaceDesc) throws TIOError, TException {
     try {
       NamespaceDescriptor descriptor = namespaceDescriptorFromThrift(namespaceDesc);
       connectionCache.getAdmin().createNamespace(descriptor);
@@ -766,8 +779,7 @@ public class ThriftHBaseServiceHandler extends HBaseServiceHandler implements TH
   }
 
   @Override
-  public void modifyNamespace(TNamespaceDescriptor namespaceDesc)
-      throws TIOError, TException {
+  public void modifyNamespace(TNamespaceDescriptor namespaceDesc) throws TIOError, TException {
     try {
       NamespaceDescriptor descriptor = namespaceDescriptorFromThrift(namespaceDesc);
       connectionCache.getAdmin().modifyNamespace(descriptor);
@@ -786,20 +798,10 @@ public class ThriftHBaseServiceHandler extends HBaseServiceHandler implements TH
   }
 
   @Override
-  public TNamespaceDescriptor getNamespaceDescriptor(String name)
-      throws TIOError, TException {
+  public TNamespaceDescriptor getNamespaceDescriptor(String name) throws TIOError, TException {
     try {
-      return namespaceDescriptorFromHBase(connectionCache.getAdmin().getNamespaceDescriptor(name));
-    } catch (IOException e) {
-      throw getTIOError(e);
-    }
-  }
-
-  @Override
-  public List<TNamespaceDescriptor> listNamespaceDescriptors()
-      throws TIOError, TException {
-    try {
-      return namespaceDescriptorsFromHBase(connectionCache.getAdmin().listNamespaceDescriptors());
+      NamespaceDescriptor descriptor = connectionCache.getAdmin().getNamespaceDescriptor(name);
+      return namespaceDescriptorFromHBase(descriptor);
     } catch (IOException e) {
       throw getTIOError(e);
     }
@@ -807,15 +809,16 @@ public class ThriftHBaseServiceHandler extends HBaseServiceHandler implements TH
 
   @Override
   public List<String> listNamespaces() throws TIOError, TException {
-    List<String> result = new ArrayList<>();
     try {
-      for (NamespaceDescriptor nd : connectionCache.getAdmin().listNamespaceDescriptors()) {
-        result.add(nd.getName());
+      String[] namespaces = connectionCache.getAdmin().listNamespaces();
+      List<String> result = new ArrayList<>(namespaces.length);
+      for (String ns: namespaces) {
+        result.add(ns);
       }
+      return result;
     } catch (IOException e) {
       throw getTIOError(e);
     }
-    return result;
   }
 
   @Override
@@ -824,14 +827,38 @@ public class ThriftHBaseServiceHandler extends HBaseServiceHandler implements TH
   }
 
   @Override
-  public List<TOnlineLogRecord> getSlowLogResponses(Set<TServerName> serverNames,
-      TLogQueryFilter logQueryFilter) throws TIOError, TException {
-    throw new NotImplementedException();
+  public List<TOnlineLogRecord> getSlowLogResponses(Set<TServerName> tServerNames,
+      TLogQueryFilter tLogQueryFilter) throws TIOError, TException {
+    try {
+      Set<ServerName> serverNames = ThriftUtilities.getServerNamesFromThrift(tServerNames);
+      LogQueryFilter logQueryFilter =
+        ThriftUtilities.getSlowLogQueryFromThrift(tLogQueryFilter);
+      List<OnlineLogRecord> onlineLogRecords =
+        connectionCache.getAdmin().getSlowLogResponses(serverNames, logQueryFilter);
+      return ThriftUtilities.getSlowLogRecordsFromHBase(onlineLogRecords);
+    } catch (IOException e) {
+      throw getTIOError(e);
+    }
   }
 
   @Override
-  public List<Boolean> clearSlowLogResponses(Set<TServerName> serverNames)
+  public List<Boolean> clearSlowLogResponses(Set<TServerName> tServerNames)
       throws TIOError, TException {
-    throw new NotImplementedException();
+    Set<ServerName> serverNames = ThriftUtilities.getServerNamesFromThrift(tServerNames);
+    try {
+      return connectionCache.getAdmin().clearSlowLogResponses(serverNames);
+    } catch (IOException e) {
+      throw getTIOError(e);
+    }
+  }
+
+  @Override
+  public List<TNamespaceDescriptor> listNamespaceDescriptors() throws TIOError, TException {
+    try {
+      NamespaceDescriptor[] descriptors = connectionCache.getAdmin().listNamespaceDescriptors();
+      return namespaceDescriptorsFromHBase(descriptors);
+    } catch (IOException e) {
+      throw getTIOError(e);
+    }
   }
 }
